#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com


from eucaops import Eucaops
from imageutils import ImageUtils
from eutester.sshconnection import CommandExitCodeException
from eutester.eutestcase import EutesterTestCase
from testcases.cloud_user.images.imageutils import ImageUtils
from testcases.cloud_user.images.conversiontask import ConversionTask
from boto.exception import S3ResponseError
from subprocess import CalledProcessError
from argparse import ArgumentError
import os
import time
import types


class ImportInstanceTests(EutesterTestCase):


    def __init__(self, url=None, tester=None, **kwargs):
        self.setuptestcase()
        self.setup_parser(testname='import_instance_tests',
                              description='Runs tests against import instance'
                                          'conversion tasks',
                              emi=False)

        self.parser.add_argument('--url',
                                 help='URL containing remote image to create '
                                      'import instance task from',
                                 default=None)
        self.parser.add_argument('--workerip',dest='worker_machine',
                                 help='The ip/hostname of the machine that the '
                                      'operation will be performed on',
                                 default=None)
        self.parser.add_argument('--worker_username',dest='worker_username',
                                 help='The username of the machine that the '
                                      'operation will be performed on, '
                                      'default:"root"',
                                 default='root')
        self.parser.add_argument('--worker_password',dest='worker_password',
                                 help='The password of the machine that the '
                                      'operation will be performed on',
                                 default=None)
        self.parser.add_argument('--worker_keypath',dest='worker_keypath',
                                 help='The ssh keypath of the machine that '
                                      'the operation will be performed on',
                                 default=None)
        self.parser.add_argument('--destpath',
                                 help='The path on the workip, that this '
                                      'operation will be performed on',
                                 default='/disk1/storage')
        self.parser.add_argument('--imagelocation',
                                 help='The file path on the worker of a '
                                      'pre-existing image to import',
                                 default=None)
        self.parser.add_argument('--urlpass', dest='wget_password',
                                 help='Password needed to retrieve remote url',
                                 default=None)
        self.parser.add_argument('--urluser', dest='wget_user',
                                 help='Username needed to retrieve remote url',
                                 default=None)
        self.parser.add_argument('--gigtime',dest='time_per_gig',
                                 help='Time allowed per gig size of image to '
                                      'be used',
                                 default=300)
        self.parser.add_argument('--virtualization_type',
                                 help='virtualization type, hvm or pv',
                                 default='hvm')
        self.parser.add_argument('--bucket', dest='bucketname',
                                 help='bucket name to be used for import task',
                                 default=None)
        self.parser.add_argument('--arch', dest='arch',
                                 help='Image architecture, '
                                      'ie:x86_64 (default), i386',
                                 default="x86_64")
        self.parser.add_argument('--imageformat',
                                 dest='imageformat',
                                 help='image format for import task ("vmdk", '
                                      '"raw", or "vhd")default, "raw"',
                                 default='raw')
        self.parser.add_argument('--platform', dest='platform',
                                 help='"Linux" or "Windows", default: "linux"',
                                 default="Linux")
        self.parser.add_argument('--uploaded_manifest', dest='upload_manifest',
                                 help='bucket/prefix location of manifest to '
                                      'register',
                                 default=None)
        self.parser.add_argument('--bundle_manifest', dest='bundle_manifest',
                                 help='file path on worker to bundle manifest '
                                      'to upload',
                                 default=None)
        self.parser.add_argument('--overwrite',
                                 help='Will overwrite files in matching work '
                                      'dir on worker machine if found',
                                 action='store_true', default=False)
        self.parser.add_argument('--time_per_gig',
                                 help='Time allowed (in addition to "base '
                                      'timeout") per image size in GB before '
                                      'timing out task. Default:100 seconds',
                                 default=100)
        self.parser.add_argument('--base_timeout',
                                 help='Base timeout value prior to adding '
                                      'time per gig of image size. '
                                      'Default:600 seconds',
                                 default=600)
        self.parser.add_argument('--no_clean_on_exit',
                                 help='Disable cleanup method upon exit to '
                                      'leave test resources behind',
                                 action='store_true', default=False)

        self.tester = tester
        self.url = url
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        # Setup basic eutester object
        if not self.tester:
            try:
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, ' \
                                'or config_file and password was provided, err:' + str(e))
            #replace default eutester debugger with eutestcase's for more verbosity...
            self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        if not self.url:
            if not self.args.url:
                raise ArgumentError(None,'Required URL not provided')
            else:
                self.url = self.args.url

        self.args.worker_password = self.args.worker_password or self.args.password
        self.args.worker_keypath = self.args.worker_keypath or self.args.keypair
        self.latest_task_dict = None
        #Create an ImageUtils helper from the arguments provided in this self...
        self.img_utils = self.do_with_args(ImageUtils)
        assert isinstance(self.tester, Eucaops)
        assert isinstance(self.img_utils, ImageUtils)
        self.setup()

    def setup(self):
        self.status("STARTING TEST SETUP...")
        self.get_import_bucket_to_use()
        self.get_source_volume_image()
        self.get_security_group()
        self.get_keypair()
        self.get_zone()
        self.status("TEST SETUP COMPLETE")

    @classmethod
    def assertEquals(cls, x, y):
        assert x == y, str(x) + ' is not equal to ' + str(y)
    @property
    def keyname(self):
        if not self.keypair:
            self.get_keypair()
        return self.keypair.name

    @property
    def groupname(self):
        if not self.group:
            self.get_security_group()
        return self.group.name

    def get_security_group(self, group_name=None):
        group_name = group_name or 'import_instance_test_group'
        tester = self.tester
        assert isinstance(tester, Eucaops)
        self.group = tester.add_group(group_name=group_name)
        #authorize group for ssh and icmp
        tester.authorize_group(self.group)
        tester.authorize_group(self.group, protocol='icmp', port='-1')
        return self.group

    def get_keypair(self):
        tester = self.tester
        assert isinstance(tester, Eucaops)
        keys = tester.get_all_current_local_keys()
        if keys:
            self.keypair = keys[0]
            self.debug('Found an existing local keypair:' + str(self.keypair))
        else:
            default_keyname = 'import_instance_test_key'+str(int(time.time()))
            try:
                local_keypath = default_keyname + ".pem"
                tester.local('ls ' + local_keypath)
                tester.local('mv ' + local_keypath + ' ' +
                             local_keypath + '_old')
            except CalledProcessError:
                pass
            self.keypair = tester.add_keypair(default_keyname)
        return self.keypair

    def get_zone(self):
        self.zone = self.args.zone
        if not self.zone:
            zones = self.tester.get_zones()
            if zones:
                self.zone = zones[0]
        if not self.zone:
            raise RuntimeError('Could not find zone to use in test')
        return self.zone

    def get_source_volume_image(self, url=None, img_utils=None):
        url = url or self.url
        img_utils = img_utils or self.img_utils
        if self.args.imagelocation:
            self.imagelocation = self.args.imagelocation
        else:
            assert isinstance(img_utils, ImageUtils)
            worker = img_utils.worker_machine
            src_img = os.path.basename(url)
            src_img = os.path.join(self.args.destpath, src_img)
            try:
                #Looking for existing file at destpath
                worker.sys('ls ' + src_img, code=0)
            except CommandExitCodeException:
                #File not found at destpath download it...
                worker.wget_remote_image(url=url, dest_file_name=src_img)
            self.imagelocation = src_img
        return self.imagelocation

    def get_import_bucket_to_use(self, bucketname=None):
        bucketname = bucketname or self.args.bucketname
        if not bucketname:
            bucketname = 'import_instance_test_bucket'
        self.bucket = self.tester.s3.create_bucket(bucketname).name

    def test1_basic_create_import_instance(self,
                                          base_timout=None,
                                          time_per_gig=None):
        '''
        Definition: Attempts to run, monitor and validate the outcome of a
        basic import instance task.
        Will test the following:

        ## TASK CHECKS:
        -Euca2ools import task request
        -Will monitor task and describe task responses until complete, or a
         given timeout is reached.
        -Upon completion will validate task status

        ## TASK INSTANCE CHECKS:
        -Instance status
        -Will Monitor instance to running, then attempt to ping and ssh.
        -Instance params (security group, key, zone, etc)
        -Will request the instance from the system to confirm it is visible
        to this account, etc..

        ## TASK VOLUME CHECKS:
        -Volume status as created and available post task
        -Request the volume from the system to confirm it is visible to this
        account, etc..
        -Volume Params are correct, size, zone, etc per task request

        ## TASK SNAPSHOT CHECKS:
        -Snapshot status completed
        -Verify the owner id is the same as the account id that made the task
        request

        ## TASK IMAGE CHECKS:
        -Verify the image is not public
        -Verify the image state is 'available'
        -Verify the image owner id is the same as the account id that made the
        task request.
        '''
        base_timout = base_timout or self.args.base_timeout
        time_per_gig = time_per_gig or self.args.time_per_gig
        img_utils = self.img_utils
        tester = self.tester
        assert isinstance(img_utils, ImageUtils)
        assert isinstance(tester, Eucaops)
        params = {'import_file':self.imagelocation,
                  'bucket':self.bucket,
                  'zone':self.zone,
                  'format':self.args.imageformat,
                  'instance_type':self.args.vmtype,
                  'arch':self.args.arch,
                  'keypair':self.keyname,
                  'group':self.groupname,
                  'platform':self.args.platform}
        task = img_utils.euca2ools_import_instance(**params)
        assert isinstance(task,ConversionTask)
        tester.monitor_conversion_tasks(task,
                                        base_timeout=base_timout,
                                        time_per_gig=time_per_gig)

        # Make sure the task returns an instance, check that the instance for
        # proper run state and param use.
        if not task.instanceid:
            raise RuntimeError('Instance ID not found after task completed, '
                               'status msg:' + str(task.statusmessage))
        inst = tester.get_instances(idstring=task.instanceid)
        if inst:
            inst = inst[0]
            username = self.args.instance_user
            euinst = tester.convert_instance_to_euisntance(instance=inst,
                                                           keypair=self.keypair,
                                                           username=username)
            tester.monitor_euinstances_to_running(euinst)
        else:
            raise RuntimeError('Instance:"{0}" not found from task:"{1}"'
                            .format(task.instanceid, task.id))
        if not task.image_id:
            raise RuntimeError('Image Id not found after task completed, '
                               'status msg:' + str(task.statusmessage))
        emi = tester.get_emi(emi=task.image_id)
        self.assertEquals(emi.owner_id, tester.get_account_id())
        for snap in task.snapshots:
            self.assertEquals(snap.owner_id, tester.get_account_id())

        self.latest_task_dict = {'params': params,
                                 'task': task,
                                 'instance': euinst}
        return self.latest_task_dict

    def test2_validate_params_against_task(self):
        if not self.latest_task_dict:
            raise RuntimeError('Dict for latest task not found to validate?')
        params = self.latest_task_dict['params']
        task = self.latest_task_dict['task']
        return self.validate_params_against_task(params=params, task=task)

    def validate_params_against_task(self, params, task):
        assert isinstance(params, types.DictionaryType)
        assert isinstance(task, ConversionTask)
        err_msg = ""
        try:
            if params.has_key('bucket'):
                self.debug('Checking task for bucket...')
                #todo put bucket checks here
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('zone'):
                zone = params['zone']
                self.debug('Checking task for zone params:' + str(zone))
                self.assertEquals(zone, task.availabilityzone)
                for volume in task.volumes:
                    self.assertEquals(volume.zone, zone)
                    self.assertEquals(task.instance.placement, zone)
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('size'):
                size = params['size']
                self.debug('Checking task for size params:' + str(size))
                self.assertEquals(zone, task.availabilityzone)
                for im_volume in task.importvolumes:
                    self.assertEquals(str(im_volume.volume.size), str(size))
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('format'):
                image_format = params['format']
                self.debug('Checking task for format:' + str(image_format))
                for volume in task.importvolumes:
                        self.assertEquals(str(volume.image.format).upper(),
                                          str(image_format).upper())
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('instance_type'):
                instance_type = params['instance_type']
                self.debug('Checking task for instance_type:' + str(instance_type))
                self.assertEquals(task.instance.instance_type, instance_type)
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('arch'):
                arch = params['arch']
                self.debug('Checking task for arch:' + str(arch))
                emi = self.tester.get_emi(emi=task.image_id)
                self.assertEquals(emi.architecture, arch)
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('keypair'):
                keypair = params['keypair']
                self.debug('Checking task for keypair:' + str(keypair))
                self.assertEquals(keypair, task.instance.key_name)
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('group'):
                group = params['group']
                self.debug('Checking task for group:' + str(group))
                ins = self.tester.convert_instance_to_euisntance(task.instance,
                                                                 auto_connect=False)
                groups = self.tester.get_instance_security_groups(ins)
                sec_group = groups[0]
                self.assertEquals(sec_group.name, group)
        except Exception as e:
            self.debug(self.tester.get_traceback())
            err_msg += str(e) + "\n"

        try:
            if params.has_key('platform'):
                platform = params['platform']
                self.debug('Checking task for platform: ' + str(platform))
                platform = str(platform).lower()
                if platform == 'linux':
                    platform = None
                self.assertEquals(platform, task.instance.platform)
        except Exception as e:
            err_msg += str(e) + "\n"

        if err_msg:
            raise Exception("Failures in param validation detected:n\n"
                            + str(err_msg))


    def clean_method(self):
        tester = self.tester
        assert isinstance(tester, Eucaops)
        tester.cleanup_artifacts()

if __name__ == "__main__":
    testcase = ImportInstanceTests()
    if testcase.args.tests:
        list = testcase.args.tests.splitlines(',')
    else:
        list = ['test1_basic_create_import_instance',
                'test2_validate_params_against_task']

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append(testcase.create_testunit_by_name(test))
    if testcase.args.no_clean_on_exit:
        clean_on_exit = False
    else:
        clean_on_exit = True

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,
                                         eof=False,
                                         clean_on_exit=clean_on_exit)
    exit(result)






# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on Mar 7, 2012
@author: clarkmatthew
Place holder class to provide convenience for testing, modifying, and
retrieving Eucalyptus cloud property information
Intention is to reduce the time in looking up property names, and values
outside of the eutester test lib, etc
Note: Debug output for the tester.sys command are controled by the
eutester/eucaops object

Sample:
cat my_cloud.conf
>   192.168.1.76	CENTOS	6.3	64	REPO	[CLC WS]
>   192.168.1.77	CENTOS	6.3	64	REPO	[SC00 CC00]
>   192.168.1.78	CENTOS	6.3	64	REPO	[NC00]


    from eucaops import Eucaops
    from eutester import euproperties
    Eucaops(config_file='my_cloud.conf', password='mypassword')

    ep_mgr = euproperties.Euproperty_Manager(tester,
                                             verbose=True,
                                             debugmethod=tester.debug)


#get some storage service properties, and some property values...

    #Get/Set value from dynamic method created in Euproperty_Manager...
        san_host_prop_value = ep_mgr.get_storage_sanhost_value()
        ep_mgr.set_storage_sanhost_value('192.168.1.200')

    #Get/set value from euproperty directly...
        san_host_prop = ep_mgr.get_property('san_host', 'storage', 'PARTI00')
        san_host_prop_value = san_host_prop.get()
        san_host_prop_set('192.168.1.200'

    #Get multiple properties at once based on certain filters...
        storage_properties = ep_mgr.get_properties(service_type='storage')
        partition1_properties = ep_mgr.get_properties(partition='partition1')

'''

import types
import re
import copy


class Euproperty_Type():
    authentication = 'authentication'
    autoscaling = 'autoscaling'
    bootstrap = 'bootstrap'
    cloud = 'cloud'
    cloudwatch = 'cloudwatch'
    cluster = 'cluster'
    dns = 'dns'
    imaging = 'imaging'
    loadbalancing = 'loadbalancing'
    objectstorage = 'objectstorage'
    reporting = 'reporting'
    storage = 'storage'
    system = 'system'
    tagging = 'tagging'
    tokens = 'tokens'
    vmwarebroker = 'vmwarebroker'
    walrus = 'walrus'
    www = 'www'

    @classmethod
    def get_type_by_string(cls, typestring):
        try:
            if hasattr(cls, str(typestring)):
                return getattr(cls, str(typestring))
        except AttributeError, ae:
            print ('Property type:' + str(str) +
                   " not defined, new property type?")
            raise ae


class Euproperty():
    def __init__(self, prop_mgr, property_string, service_type, partition,
                 name, value, mandatory=False, description=""):
        self.prop_mgr = prop_mgr
        self.service_type = Euproperty_Type.get_type_by_string(service_type)
        self.partition = partition
        self.name = name
        self.value = value
        self.property_string = property_string
        self.prop_mgr = prop_mgr
        self.lastvalue = value
        self.mandatory = mandatory

    def update(self):
        newprop = self.prop_mgr.update_property_list(
            property_name=self.property_string)[0]
        self = newprop

    def get(self):
        return self.value

    def set(self, value):
        return self.prop_mgr.set_property(self, value)

    def reset_to_default(self):
        return self.prop_mgr.reset_property_to_default(self)

    def print_self(self, include_header=True, show_description=True,
                   print_method=None, printout=True):
        if printout and not print_method:
            print_method = self.prop_mgr.debug
        name_len = 50
        service_len = 20
        part_len = 20
        value_len = 30
        line_len = 120
        ret = ""

        header = str('NAME').ljust(name_len)
        header += "|" + str('SERVICE TYPE').center(service_len)
        header += "|" + str('PARTITION').center(part_len)
        header += "|" + str('VALUE').center(value_len)
        header += "\n"

        out = str(self.name).ljust(name_len)
        out += "|" + str(self.service_type).center(service_len)
        out += "|" + str(self.partition).center(part_len)
        out += "|" + str(self.value).center(value_len)
        out += "\n"
        line = "-"
        for x in xrange(0, line_len):
            line += "-"
        line += "\n"
        if include_header:
            ret = "\n" + line + header + line
        ret += out
        if show_description:
            ret += "DESCRIPTION: " + self.description + "\n"
        ret += line
        if print_method:
            print_method(ret)
        return ret


class Property_Map():
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


class Euproperty_Manager():
    tester = None
    verbose = False
    debugmethod = None

    def __init__(self, tester, verbose=False, machine=None,
                 service_url=None, debugmethod=None):
        self.tester = tester
        self.debugmethod = debugmethod or tester.debug
        self.verbose = verbose
        self.work_machine = machine or self.get_clc()
        self.access_key = self.tester.aws_access_key_id
        self.secret_key = self.tester.aws_secret_access_key
        self.service_url = service_url or str(
            'http://' + str(self.get_clc().hostname) +
            ':8773/services/Eucalytpus')
        self.cmdpath = self.tester.eucapath+'/usr/sbin/'
        self.properties = []
        self.property_map = Property_Map()
        self.update_property_list()
        self.tester.property_manager = self
        self.zones = self.tester.get_zones()

    def get_clc(self):
        return self.tester.service_manager.get_enabled_clc().machine

    def debug(self, msg):
        '''
        simple method for printing debug.
        msg - mandatory - string to be printed
        method - optional - callback to over ride default printing method
        '''
        if (self.debugmethod is None):
            print (str(msg))
        else:
            self.debugmethod(msg)

    def show_all_authentication_properties(self,
                                           partition=None,
                                           debug_method=None):
        return self.show_all_properties(
            service_type=Euproperty_Type.authentication,
            partition=partition,
            debug_method=debug_method,
            descriptions=True)

    def show_all_bootstrap_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.bootstrap,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_cloud_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.cloud,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_cluster_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.cluster,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_reporting_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.reporting,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_storage_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.storage,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_system_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.system,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_vmwarebroker_properties(self,
                                         partition=None,
                                         debug_method=None):
        return self.show_all_properties(
            service_type=Euproperty_Type.vmwarebroker,
            partition=partition,
            debug_method=debug_method,
            descriptions=True)

    def show_all_walrus_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.walrus,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_www_properties(self,  partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.www,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_autoscaling_properties(self,
                                        partition=None,
                                        debug_method=None):
        return self.show_all_properties(
            service_type=Euproperty_Type.autoscaling,
            partition=partition,
            debug_method=debug_method,
            descriptions=True)

    def show_all_loadbalancing_properties(self,
                                          partition=None,
                                          debug_method=None):
        return self.show_all_properties(
            service_type=Euproperty_Type.loadbalancing,
            partition=partition,
            debug_method=debug_method,
            descriptions=True)

    def show_all_tagging_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.tagging,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_imaging_properties(self, partition=None, debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.imaging,
                                        partition=partition,
                                        debug_method=debug_method,
                                        descriptions=True)

    def show_all_properties(self,
                            partition=None,
                            service_type=None,
                            value=None,
                            search_string=None,
                            list=None,
                            debug_method=None,
                            descriptions=True):
        debug_method = debug_method or self.debug
        list = list or self.get_properties(partition=partition,
                                           service_type=service_type,
                                           value=value,
                                           search_string=search_string)
        first = list.pop(0)
        buf = first.print_self(include_header=True,
                               show_description=descriptions,
                               printout=False)
        count = 1
        last_service_type = first.service_type
        for prop in list:
            count += 1
            if prop.service_type != last_service_type:
                last_service_type = prop.service_type
                print_header = True
            else:
                print_header = False
            buf += prop.print_self(include_header=print_header,
                                   show_description=descriptions,
                                   printout=False)
        debug_method(buf)

    def get_properties(self,
                       partition=None,
                       service_type=None,
                       value=None,
                       search_string=None,
                       force_update=False):
        self.debug('get_properties: partition:' +
                   str(partition) + ", service_type:" + str(service_type) +
                   ", value:" + str(value) + ", force_update:" +
                   str(force_update))
        ret_props = []
        if not self.properties or force_update:
            self.update_property_list()
        properties = copy.copy(self.properties)
        if partition and properties:
            properties = self.get_all_properties_for_partition(partition,
                                                               list=properties)
        if service_type and properties:
            properties = self.get_all_properties_for_service(service_type,
                                                             list=properties)
        if search_string and properties:
            properties = self.get_all_properties_by_search_string(
                search_string, list=properties)
        if properties:
            if value:
                for prop in properties:
                    if prop.value == value:
                        ret_props.append(prop)
            else:
                ret_props.extend(properties)
        return ret_props

    def get_property(self, name, service_type, partition, force_update=False):
        self.debug('Get Property:' + str(name))
        ret_prop = None
        list = self.get_properties(partition=partition,
                                   service_type=service_type,
                                   force_update=force_update)
        if list:
            ret_prop = self.get_euproperty_by_name(name, list=list)
        return ret_prop

    def update_property_list(self, property_name=''):
        newlist = []
        newprop = None
        self.debug("updating property list...")
        self.zones = self.tester.get_zones()
        cmdout = self.work_machine.sys(
            self.cmdpath+'euca-describe-properties -v -U ' +
            str(self.service_url) + ' -I ' + str(self.access_key) +
            ' -S ' + str(self.secret_key) + ' ' + property_name,
            code=0, verbose=self.verbose)
        for propstring in cmdout:
            try:
                if re.search("^PROPERTY", propstring):
                    newprop = self.parse_euproperty_from_string(propstring)
                elif newprop:
                    if (re.search("^DESCRIPTION", propstring) and
                            re.search(newprop.name, propstring)):
                        newprop.description = \
                            self.parse_euproperty_description(propstring)
                    else:
                        newprop.value = str(newprop.value) + str(propstring)
            except Exception, e:
                self.debug('Error processing property line: ' + propstring)
                raise e
            if not newprop in newlist:
                newlist.append(newprop)
        if property_name:
            for newprop in newlist:
                for oldprop in self.properties:
                    if oldprop.property_string == newprop.property_string:
                        oldprop = newprop
                        self.create_dynamic_property_map_from_property(newprop)
        else:
            self.properties = newlist
            self.property_map = Property_Map()
            for prop in self.properties:
                self.create_dynamic_property_map_from_property(prop)
        return newlist

    def parse_euproperty_description(self, propstring):
        '''
        Example string to parse:
        "DESCRIPTION	www.http_port	Listen to HTTP on this port."
        '''
        split = str(propstring).replace('DESCRIPTION', '').split()
        description = " ".join(str(x) for x in split[1:])
        return str(description)

    def parse_property_value_from_string(self, propstring):
        split = str(propstring).replace('PROPERTY', '').split()
        prop_value = " ".join(str(x) for x in split[1:])
        return str(prop_value)

    def parse_euproperty_from_string(self, propstring):
        '''
        Intended to convert a line of ouptut from euca-describe-properties into
        a euproperty.
        :param str: line of output, example:
        "PROPERTY    walrus.storagemaxbucketsizeinmb    5120"
        :returns euproperty
        '''
        propstring = str(propstring).replace('PROPERTY', '').strip()
        ret_service_type = None
        ret_partition = None
        splitstring = propstring.split()
        #get the property string, example: "walrus.storagemaxbucketsizeinmb"
        property_string = splitstring.pop(0)
        ret_value = " ".join(splitstring)
        for prop in self.properties:
            #if this property is in our list, update the value and return
            if prop.property_string == property_string:
                prop.lastvalue = prop.value
                prop.value = ret_value
                return prop
        ret_name = property_string
        #...otherwise this property is not in our list yet,
        # create a new property
        #parse property string into values...
        propattrs = property_string.split('.')

        #See if the first element is a zone-partition
        #First store and remove the zone-partition if it's in the list
        for zone in self.zones:
            if zone == propattrs[0]:
                #Assume this is the zone-partition id/name,
                # remove it from the propattrs list
                ret_partition = propattrs.pop(0)
                break
        #Move along items in list until we reach a service type
        for index in xrange(0, len(propattrs)):
            try:
                ret_service_type = Euproperty_Type.get_type_by_string(
                    propattrs[index])
                propattrs.remove(propattrs[index])
                break
            except AttributeError:
                pass
            except IndexError:
                self.debug("\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
                           "!!!!!!!!!!!!!!!!!!!!!!!!!\n" +
                           "Need to add new service? " +
                           "No service type found for: " +
                           str(property_string) +
                           "\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
                           "!!!!!!!!!!!!!!!!!!!!!!!!!\n")
                ret_service_type = propattrs.pop(0)

        #self.debug("ret_service_type: "+str(ret_service_type))

        #Store the name of the property
        ret_name = ".".join(propattrs)
        newprop = Euproperty(self, property_string, ret_service_type,
                             ret_partition, ret_name, ret_value)
        return newprop

    def create_dynamic_property_map_from_property(self, euproperty):
        context = self.property_map
        if not hasattr(context, 'all'):
            setattr(context, 'all', Property_Map())
        all_map = getattr(context, 'all')
        if euproperty.partition:
            if not hasattr(context, str(euproperty.partition)):
                setattr(context, str(euproperty.partition), Property_Map())
            context = getattr(context, str(euproperty.partition))
        if euproperty.service_type:
            if not hasattr(context, str(euproperty.service_type)):
                setattr(context, str(euproperty.service_type), Property_Map())
            context = getattr(context, str(euproperty.service_type))
        object_name = str(euproperty.name).replace('.', '_')
        if not hasattr(context, object_name):
            setattr(context, object_name, euproperty)
        if not hasattr(all_map, object_name):
            setattr(all_map, object_name, euproperty)

    def get_euproperty_by_name(self, name, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if property.name == name:
                return property
        raise EupropertyNotFoundException('Property not found by name:' +
                                          str(name))

    def get_all_properties_for_partition(self,
                                         partition,
                                         list=None,
                                         verbose=False):
        self.debug('Get all properties for partition:' + str(partition))
        props = []
        list = list or self.properties
        for property in list:
            if property.partition == partition:
                if verbose:
                    self.debug('property:' + str(property.name) +
                               ", prop.partition:" + str(property.partition) +
                               ",partition:" + str(partition))
                props.append(property)
        self.debug('Returning list of len:' + str(len(props)))
        return props

    def get_all_properties_for_service(self, service, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if property.service_type == service:
                props.append(property)
        return props

    def get_all_properties_by_search_string(self, search_string, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if re.search(search_string, property.property_string):
                props.append(property)
        return props

    def set_property(self, property, value):
        if isinstance(property, Euproperty):

            return self.set_property_by_property_string(
                property.property_string, value)
        else:
            return self.set_property_by_property_string(str(property), value)

    def set_property(self, property, value, reset_to_default=False):
        '''
        Sets the property 'prop' at eucaops/eutester object 'tester' to 'value'
        Returns new value
        prop - mandatory - str representing the property to set
        value - mandatory - str representing the value to set the property to
        eucaops - optional - the eucaops/eutester object to set the property at
        '''
        value = str(value)
        if not isinstance(property, Euproperty):
            try:
                property = self.get_all_properties_by_search_string(property)
                if len(property) > 1:
                    raise Exception('More than one euproperty found for '
                                    'property string:' + str(property))
                else:
                    property = property[0]
            except Exception, e:
                raise Exception('Could not fetch property to set. '
                                'Using string:' + str(property))
        property.lastvalue = property.value
        self.debug('Setting property(' + property.property_string +
                   ') to value:' + str(value))
        if reset_to_default:
            ret_string = self.work_machine.sys(
                self.cmdpath + 'euca-modify-property -U ' +
                str(self.service_url) + ' -I ' + str(self.access_key) +
                ' -S ' + str(self.secret_key) + ' -r ' +
                str(property.property_string), code=0)[0]
        else:
            ret_string = self.work_machine.sys(
                self.cmdpath + 'euca-modify-property -U ' +
                str(self.service_url) + ' -I '+str(self.access_key) + ' -S ' +
                str(self.secret_key) + ' -p ' +
                str(property.property_string) + '=' + str(value),
                code=0)[0]
        if ret_string:
            ret_value = str(ret_string).split()[2]
        else:
            raise EupropertiesException("set_property output from modify "
                                        "was None")
        #Confirm property value was set
        if not reset_to_default and (ret_value != value) and\
                not (not value and ret_value == '{}'):
            ret_string = "\n".join(str(x) for x in ret_string)
            raise EupropertiesException(
                "set property(" + property.property_string + ") to value(" +
                str(value) + ") failed.Ret Value (" + str(ret_value) +
                ")\nRet String\n" + ret_string)
        property.value = ret_value
        return ret_value

    def get_property_by_string(self, property_string):
        property = None
        for prop in self.properties:
            if prop.property_string == property_string:
                property = prop
                break
        return property

    def set_property_value_by_string(self, property_string, value):
        property = self.get_property_by_string(property_string)
        if not property:
            raise Exception('Property not found for:' + str(property_string))
        property.set(value)

    def get_property_value_by_string(self, property_string):
        property = self.get_property_by_string(property_string)
        if not property:
            raise Exception('Property not found for:' + str(property_string))
        return property.value

    def reset_property_to_default(self, prop):
        '''
        Sets a property 'prop' at eucaops/eutester object 'eucaops' to it's
        default value
        Returns new value
        prop - mandatory - string representing the property to set
        ucaops - optional - the eucaops/eutester object to set the property at
        '''
        if not isinstance(prop, Euproperty):
            prop = self.get_all_properties_by_search_string(prop)[0]
        return self.set_property(prop, None, reset_to_default=True)

    def get_property_default_value(self, prop, ireadthewarning=False):
        '''
        Note: This hack method is intrusive! It will briefly reset the property
        This is a temporary method to get a properties default method
        prop - mandatory - string, eucalyptus property
        ireadthewarning - mandatory - boolean, to warn user this method
        is intrusive
        '''
        if (ireadthewarning is False):
            raise EupropertiesException("ireadthewarning is set to false in "
                                        "get_property_default_value")
        original = prop.get()
        default = self.reset_property_to_default(prop)
        prop.set(original)
        return default


class EupropertiesException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class EupropertyNotFoundException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: tony@eucalyptus.com
import re
import copy

import boto
from boto.ec2.autoscale import ScalingPolicy, Instance
from boto.ec2.autoscale import Tag
from boto.ec2.autoscale import LaunchConfiguration
from boto.ec2.autoscale import AutoScalingGroup
from boto.ec2.regioninfo import RegionInfo

from eutester import Eutester


ASRegionData = {
    'us-east-1': 'autoscaling.us-east-1.amazonaws.com',
    'us-west-1': 'autoscaling.us-west-1.amazonaws.com',
    'us-west-2': 'autoscaling.us-west-2.amazonaws.com',
    'eu-west-1': 'autoscaling.eu-west-1.amazonaws.com',
    'ap-northeast-1': 'autoscaling.ap-northeast-1.amazonaws.com',
    'ap-southeast-1': 'autoscaling.ap-southeast-1.amazonaws.com',
    'ap-southeast-2': 'autoscaling.ap-southeast-2.amazonaws.com',
    'sa-east-1': 'autoscaling.sa-east-1.amazonaws.com'}


class ASops(Eutester):
    def __init__(self, host=None, credpath=None, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None,
                 username="root", region=None, is_secure=False, path='/', port=80, boto_debug=0):
        """
        :param host:
        :param credpath:
        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param username:
        :param region:
        :param is_secure:
        :param path:
        :param port:
        :param boto_debug:
        """
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.account_id = None
        self.user_id = None
        super(ASops, self).__init__(credpath=credpath)

        self.setup_as_connection(host=host,
                                 region=region,
                                 endpoint=endpoint,
                                 aws_access_key_id=self.aws_access_key_id,
                                 aws_secret_access_key=self.aws_secret_access_key,
                                 is_secure=is_secure,
                                 path=path,
                                 port=port,
                                 boto_debug=boto_debug)
        self.poll_count = 48
        self.username = username
        self.test_resources = {}
        self.setup_as_resource_trackers()

    @Eutester.printinfo
    def setup_as_connection(self, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None, is_secure=True,
                            host=None, region=None, path="/", port=443, boto_debug=0):
        """
        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param is_secure:
        :param host:
        :param region:
        :param path:
        :param port:
        :param boto_debug:
        :raise:
        """
        as_region = RegionInfo()
        if region:
            self.debug("Check region: " + str(region))
            try:
                if not endpoint:
                    as_region.endpoint = ASRegionData[region]
                else:
                    as_region.endpoint = endpoint
            except KeyError:
                raise Exception('Unknown region: %s' % region)
        else:
            as_region.name = 'eucalyptus'
            if not host:
                if endpoint:
                    as_region.endpoint = endpoint
                else:
                    as_region.endpoint = self.get_as_ip()
        connection_args = {'aws_access_key_id': aws_access_key_id,
                           'aws_secret_access_key': aws_secret_access_key,
                           'is_secure': is_secure,
                           'debug': boto_debug,
                           'port': port,
                           'path': path,
                           'region': as_region}

        if re.search('2.6', boto.__version__):
            connection_args['validate_certs'] = False
        try:
            as_connection_args = copy.copy(connection_args)
            as_connection_args['path'] = path
            as_connection_args['region'] = as_region
            self.debug("Attempting to create auto scale connection to " + as_region.endpoint + ':' + str(port) + path)
            self.autoscale = boto.ec2.autoscale.AutoScaleConnection(**as_connection_args)
        except Exception, e:
            self.critical("Was unable to create auto scale connection because of exception: " + str(e))

        #Source ip on local test machine used to reach instances
        self.as_source_ip = None

    def setup_as_resource_trackers(self):
        """
        Setup keys in the test_resources hash in order to track artifacts created
        """
        self.test_resources["keypairs"] = []
        self.test_resources["security-groups"] = []
        self.test_resources["images"] = []

    def create_launch_config(self, name, image_id, key_name=None, security_groups=None, user_data=None,
                             instance_type=None, kernel_id=None, ramdisk_id=None, block_device_mappings=None,
                             instance_monitoring=False, instance_profile_name=None):
        """
        Creates a new launch configuration with specified attributes.

        :param name: Name of the launch configuration to create. (Required)
        :param image_id: Unique ID of the Amazon Machine Image (AMI) assigned during registration. (Required)
        :param key_name: The name of the EC2 key pair.
        :param security_groups: Names of the security groups with which to associate the EC2 instances.
        """
        lc = LaunchConfiguration(name=name,
                                 image_id=image_id,
                                 key_name=key_name,
                                 security_groups=security_groups,
                                 user_data=user_data,
                                 instance_type=instance_type,
                                 kernel_id=kernel_id,
                                 ramdisk_id=ramdisk_id,
                                 block_device_mappings=block_device_mappings,
                                 instance_monitoring=instance_monitoring,
                                 instance_profile_name=instance_profile_name)
        self.debug("Creating launch config: " + name)
        self.autoscale.create_launch_configuration(lc)
        if len(self.describe_launch_config([name])) != 1:
            raise Exception('Launch Config not created')
        self.debug('SUCCESS: Created Launch Config: ' +
                   self.describe_launch_config([name])[0].name)

    def describe_launch_config(self, names=None):
        """
        return a list of launch configs

        :param names: list of names to query (optional) otherwise return all launch configs
        :return:
        """
        return self.autoscale.get_all_launch_configurations(names=names)

    def delete_launch_config(self, launch_config_name):
        self.debug("Deleting launch config: " + launch_config_name)
        self.autoscale.delete_launch_configuration(launch_config_name)
        if len(self.describe_launch_config([launch_config_name])) != 0:
            raise Exception('Launch Config not deleted')
        self.debug('SUCCESS: Deleted Launch Config: ' + launch_config_name)

    def create_as_group(self, group_name, launch_config, availability_zones, min_size, max_size, load_balancers=None,
                        desired_capacity=None, termination_policies=None, default_cooldown=None, health_check_type=None,
                        health_check_period=None, tags=None):
        """
        Create auto scaling group.

        :param group_name: Name of autoscaling group (required).
        :param load_balancers: List of load balancers.
        :param availability_zones: List of availability zones (required).
        :param launch_config: Name of launch configuration (required).
        :param min_size:  Minimum size of group (required).
        :param max_size: Maximum size of group (required).
        """
        self.debug("Creating Auto Scaling group: " + group_name)
        as_group = AutoScalingGroup(connection=self.autoscale,
                                    group_name=group_name,
                                    load_balancers=load_balancers,
                                    availability_zones=availability_zones,
                                    launch_config=launch_config,
                                    min_size=min_size,
                                    max_size=max_size,
                                    desired_capacity=desired_capacity,
                                    default_cooldown=default_cooldown,
                                    health_check_type=health_check_type,
                                    health_check_period=health_check_period,
                                    tags=tags,
                                    termination_policies=termination_policies)
        self.autoscale.create_auto_scaling_group(as_group)

        as_group = self.describe_as_group(group_name)

        self.debug("SUCCESS: Created Auto Scaling Group: " + as_group.name)
        return as_group

    def describe_as_group(self, name=None):
        """
        Returns a full description of each Auto Scaling group in the given
        list. This includes all Amazon EC2 instances that are members of the
        group. If a list of names is not provided, the service returns the full
        details of all Auto Scaling groups.
        :param name:
        :return:
        """
        groups = self.autoscale.get_all_groups(names=[name])
        if len(groups) > 1:
            raise Exception("More than one group with name: " + name)
        if len(groups) == 0:
            raise Exception("No group found with name: " + name)
        return groups[0]

    def delete_as_group(self, name=None, force=None):
        self.debug("Deleting Auto Scaling Group: " + name)
        self.debug("Forcing: " + str(force))
        # self.autoscale.set_desired_capacity(group_name=names, desired_capacity=0)
        self.autoscale.delete_auto_scaling_group(name=name, force_delete=force)
        try:
            self.describe_as_group([name])
            raise Exception('Auto Scaling Group not deleted')
        except:
            self.debug('SUCCESS: Deleted Auto Scaling Group: ' + name)

    def create_as_policy(self, name, adjustment_type, scaling_adjustment, as_name, cooldown=None):
        """
        Create an auto scaling policy

        :param name:
        :param adjustment_type: (ChangeInCapacity, ExactCapacity, PercentChangeInCapacity)
        :param scaling_adjustment:
        :param as_name:
        :param cooldown: (if something gets scaled, the wait in seconds before trying again.)
        """
        scaling_policy = ScalingPolicy(name=name,
                                       adjustment_type=adjustment_type,
                                       as_name=as_name,
                                       scaling_adjustment=scaling_adjustment,
                                       cooldown=cooldown)
        self.debug("Creating Auto Scaling Policy: " + name)
        self.autoscale.create_scaling_policy(scaling_policy)

    def describe_as_policies(self, as_group=None, policy_names=None):
        """
        If no group name or list of policy names are provided, all
        available policies are returned.

        :param as_group:
        :param policy_names:
        """
        self.autoscale.get_all_policies(as_group=as_group, policy_names=policy_names)

    def execute_as_policy(self, policy_name=None, as_group=None, honor_cooldown=None):
        self.debug("Executing Auto Scaling Policy: " + policy_name)
        self.autoscale.execute_policy(policy_name=policy_name, as_group=as_group, honor_cooldown=honor_cooldown)

    def delete_as_policy(self, policy_name=None, autoscale_group=None):
        self.debug("Deleting Policy: " + policy_name + " from group: " + autoscale_group)
        self.autoscale.delete_policy(policy_name=policy_name, autoscale_group=autoscale_group)

    def cleanup_autoscaling_groups(self, asg_list = None):
        """
        This will attempt to delete auto scaling groups listed in test_resources['auto-scaling-groups']
        """
        ### clear all ASGs
        if not asg_list:
            auto_scaling_groups=self.test_resources['auto-scaling-groups']
        else:
            auto_scaling_groups = asg_list
        for asg in auto_scaling_groups:
            self.debug("Found Auto Scaling Group: " + asg.name)
            self.delete_as_group(name=asg.name, force=True)



    def delete_all_autoscaling_groups(self):
        """
        This will attempt to delete all launch configs and all auto scaling groups
        """
        ### clear all ASGs
        for asg in self.describe_as_group():
            self.debug("Found Auto Scaling Group: " + asg.name)
            self.delete_as_group(name=asg.name, force=True)
        if len(self.describe_as_group(asg.name)) != 0:
            self.debug("Some AS groups remain")
            for asg in self.describe_as_group():
                self.debug("Found Auto Scaling Group: " + asg.name)

    def cleanup_launch_configs(self):
        """
        This will attempt to delete launch configs listed in test_resources['launch-configurations']
        """
        launch_configurations=self.test_resources['launch-configurations']

        if not launch_configurations:
            self.debug("Launch configuration list is empty")
        else:
            for lc in launch_configurations:
                self.debug("Found Launch Config:" + lc.name)
                self.delete_launch_config(lc.name)

    def delete_all_launch_configs(self):
        ### clear all LCs
        """
        Attempt to remove all launch configs
        """
        for lc in self.describe_launch_config():
            self.debug("Found Launch Config:" + lc.name)
            self.delete_launch_config(lc.name)
        if len(self.describe_launch_config()) != 0:
            self.debug("Some Launch Configs Remain")
            for lc in self.describe_launch_config():
                self.debug("Found Launch Config:" + lc.name)

    def get_as_ip(self):
        """Parse the eucarc for the AWS_AUTO_SCALING_URL"""
        as_url = self.parse_eucarc("AWS_AUTO_SCALING_URL")
        return as_url.split("/")[2].split(":")[0]

    def get_as_path(self):
        """Parse the eucarc for the AWS_AUTO_SCALING_URL"""
        as_url = self.parse_eucarc("AWS_AUTO_SCALING_URL")
        as_path = "/".join(as_url.split("/")[3:])
        return as_path

    def get_last_instance_id(self):
        reservations = self.ec2.get_all_instances()
        instances = [i for r in reservations for i in r.instances]
        newest_time = None
        newest_id = None
        for i in instances:
            if not newest_time or i.launch_time > newest_time:
                newest_time = i.launch_time
                newest_id = i.id
        return newest_id

    def create_group_tag(self, key, value, resource_id, propagate_at_launch=None):
        # self.debug("Number of tags: " + str(len(self.tester.autoscale.get_all_tags())))
        # self.debug("Autoscale group info: " + str(self.tester.autoscale.get_all_groups(names=[auto_scaling_group_name])[0].tags))

        tag = Tag(key=key, value=value, propagate_at_launch=propagate_at_launch, resource_id=resource_id)
        self.autoscale.create_or_update_tags([tag])
        if len(self.autoscale.get_all_tags(filters=key)) != 1:
            self.debug("Number of tags: " + str(len(self.autoscale.get_all_tags(filters=key))))
            raise Exception('Tag not created')
        self.debug("created or updated tag: " + str(self.autoscale.get_all_tags(filters=key)[0]))

    def delete_all_group_tags(self):
        all_tags = self.autoscale.get_all_tags()
        self.autoscale.delete_tags(all_tags)
        self.debug("Number of tags: " + str(len(self.autoscale.get_all_tags())))

    def delete_all_policies(self):
        policies = self.autoscale.get_all_policies()
        for policy in policies:
            self.delete_as_policy(policy_name=policy.name, autoscale_group=policy.as_name)
        if len(self.autoscale.get_all_policies()) != 0:
            raise Exception('Not all auto scaling policies deleted')
        self.debug("SUCCESS: Deleted all auto scaling policies")

    def update_as_group(self, group_name, launch_config, min_size, max_size, availability_zones=None,
                        desired_capacity=None, termination_policies=None, default_cooldown=None, health_check_type=None,
                        health_check_period=None):
        """

        :param group_name: REQUIRED
        :param launch_config: REQUIRED
        :param min_size: REQUIRED
        :param max_size: REQUIRED
        :param availability_zones:
        :param desired_capacity:
        :param termination_policies:
        :param default_cooldown:
        :param health_check_type:
        :param health_check_period:
        """
        self.debug("Updating ASG: " + group_name)
        return AutoScalingGroup(connection=self.autoscale,
                         name=group_name,
                         launch_config=launch_config,
                         min_size=min_size,
                         max_size=max_size,
                         availability_zones=availability_zones,
                         desired_capacity=desired_capacity,
                         default_cooldown=default_cooldown,
                         health_check_type=health_check_type,
                         health_check_period=health_check_period,
                         termination_policies=termination_policies).update()

    def wait_for_instances(self, group_name, number=1):
        asg = self.describe_as_group(group_name)
        instances = asg.instances
        if not instances:
            self.debug("No instances in ASG")
            return False
        if len(asg.instances) != number:
            self.debug("Instances not yet allocated")
            return False
        for instance in instances:
            assert isinstance(instance, Instance)
            instance = self.get_instances(idstring=instance.instance_id)[0]
            if instance.state != "running":
                self.debug("Instance: " + str(instance) + " still in " + instance.state + " state")
                return False
            else:
                self.debug("Instance: " + str(instance) + " now running")
        return True



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
import re
import copy
import time
from boto.ec2.regioninfo import RegionInfo
import boto
from concurrent.futures import ThreadPoolExecutor
import urllib2
import cookielib
from eutester import Eutester
from boto.ec2.elb.listener import Listener
from boto.ec2.elb.healthcheck import HealthCheck
from os.path import join

ELBRegionData = {
    'us-east-1': 'elasticloadbalancing.us-east-1.amazonaws.com',
    'us-west-1': 'elasticloadbalancing.us-west-1.amazonaws.com',
    'us-west-2': 'elasticloadbalancing.us-west-2.amazonaws.com',
    'eu-west-1': 'elasticloadbalancing.eu-west-1.amazonaws.com',
    'ap-northeast-1': 'elasticloadbalancing.ap-northeast-1.amazonaws.com',
    'ap-southeast-1': 'elasticloadbalancing.ap-southeast-1.amazonaws.com'}


class ELBops(Eutester):
    @Eutester.printinfo
    def __init__(self, host=None, credpath=None, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None,
                 username="root", region=None, is_secure=False, path='/', port=80, boto_debug=0):
        """

        :param host:
        :param credpath:
        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param username:
        :param region:
        :param is_secure:
        :param path:
        :param port:
        :param boto_debug:
        """
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.account_id = None
        self.user_id = None
        super(ELBops, self).__init__(credpath=credpath)

        self.setup_elb_connection(host=host,
                                  region=region,
                                  endpoint=endpoint,
                                  aws_access_key_id=self.aws_access_key_id,
                                  aws_secret_access_key=self.aws_secret_access_key,
                                  is_secure=is_secure,
                                  path=path,
                                  port=port,
                                  boto_debug=boto_debug)
        self.poll_count = 48
        self.username = username
        self.test_resources = {}
        self.setup_elb_resource_trackers()

    @Eutester.printinfo
    def setup_elb_connection(self, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None, is_secure=True,
                             host=None,
                             region=None, path="/", port=443, boto_debug=0):
        """

        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param is_secure:
        :param host:
        :param region:
        :param path:
        :param port:
        :param boto_debug:
        :raise:
        """
        elb_region = RegionInfo()
        if region:
            self.debug("Check region: " + str(region))
            try:
                if not endpoint:
                    elb_region.endpoint = ELBRegionData[region]
                else:
                    elb_region.endpoint = endpoint
            except KeyError:
                raise Exception('Unknown region: %s' % region)
        else:
            elb_region.name = 'eucalyptus'
            if not host:
                if endpoint:
                    elb_region.endpoint = endpoint
                else:
                    elb_region.endpoint = self.get_elb_ip()
        connection_args = {'aws_access_key_id': aws_access_key_id,
                           'aws_secret_access_key': aws_secret_access_key,
                           'is_secure': is_secure,
                           'debug': boto_debug,
                           'port': port,
                           'path': path,
                           'region': elb_region}

        if re.search('2.6', boto.__version__):
            connection_args['validate_certs'] = False

        try:
            elb_connection_args = copy.copy(connection_args)
            elb_connection_args['path'] = path
            elb_connection_args['region'] = elb_region
            self.debug(
                "Attempting to create load balancer connection to " + elb_region.endpoint + ':' + str(port) + path)
            self.elb = boto.connect_elb(**elb_connection_args)
        except Exception, e:
            self.critical("Was unable to create elb connection because of exception: " + str(e))

    def setup_elb_resource_trackers(self):
        """
        Setup keys in the test_resources hash in order to track artifacts created
        """
        self.test_resources["load_balancers"] = []

    def get_elb_ip(self):
        """Parse the eucarc for the AWS_ELB_URL"""
        elb_url = self.parse_eucarc("AWS_ELB_URL")
        return elb_url.split("/")[2].split(":")[0]

    def get_elb_path(self):
        """Parse the eucarc for the AWS_ELB_URL"""
        elb_url = self.parse_eucarc("AWS_ELB_URL")
        elb_path = "/".join(elb_url.split("/")[3:])
        return elb_path

    def create_listener(self, load_balancer_port=80, protocol="HTTP", instance_port=80, load_balancer=None):
        self.debug(
            "Creating ELB Listener for protocol " + protocol + " and port " + str(load_balancer_port) + "->" + str(
                instance_port))
        listner = Listener(load_balancer=load_balancer,
                           protocol=protocol,
                           load_balancer_port=load_balancer_port,
                           instance_port=instance_port)
        return listner

    def create_healthcheck(self, target="HTTP:80/instance-name", interval=10, timeout=5, healthy_threshold=2,
                           unhealthy_threshold=10):
        self.debug("Creating healthcheck: " + target + " interval=" + str(interval) + " timeout=" + str(timeout) +
                   " healthy threshold=" + str(healthy_threshold) + " unhealthy threshold=" + str(unhealthy_threshold))
        healthcheck = HealthCheck(target=target,
                                  timeout=timeout,
                                  interval=interval,
                                  healthy_threshold=healthy_threshold,
                                  unhealthy_threshold=unhealthy_threshold)
        return healthcheck

    def generate_http_requests(self, url, count=100, worker_threads=20):
        self.debug("Generating {0} http requests against {1}".format(count, url))
        jar = cookielib.FileCookieJar("cookies")
        handler = urllib2.HTTPCookieProcessor(jar)
        opener = urllib2.build_opener(handler)
        urllib2.install_opener(opener)
        response_futures = []
        with ThreadPoolExecutor(max_workers=worker_threads) as executor:
            for _ in range(count):
                response_futures.append(executor.submit(urllib2.urlopen, url))

        responses = []
        for response in response_futures:
            http_response = response.result()
            try:
                http_error_code = http_response.getcode()
                if http_error_code == 200:
                    result = "{0}".format(http_response.read().rstrip())
                    self.debug("Request response: " + result)
                    responses.append(result)
                else:
                    raise Exception("Error code " + http_error_code + " found when sending " +
                                    str(worker_threads) + " concurrent requests to " + url)
            finally:
                http_response.close()
        return responses

    def register_lb_instances(self, name, instances, timeout=360, poll_count=15):
        inst_ids = [inst.id for inst in instances]
        self.debug("Registering instances {0} with lb {1}".format(inst_ids, name))
        self.elb.register_instances(name, inst_ids)
        poll_sleep = timeout / poll_count
        for _ in range(poll_count):
            self.debug("Checking instance health for {0}".format(inst_ids))
            inst_states = self.elb.describe_instance_health(name, instances=inst_ids)
            states = [state.state for state in inst_states]
            if not states or 'OutOfService' in states:
                time.sleep(poll_sleep)
            elif 'InService' in states:
                self.debug("Instances {0} for lb {1} are InService".format(inst_ids, name))
                return
            else:
                # This should never happen
                pass
        raise Exception("Instances {0} failed to enter InService state before timeout".format(inst_ids))

    def create_load_balancer(self, zones, name="test", load_balancer_port=80, instances=None):
        self.debug("Creating load balancer: " + name + " on port " + str(load_balancer_port))
        listener = self.create_listener(load_balancer_port=load_balancer_port)
        self.elb.create_load_balancer(name, zones=zones, listeners=[listener])

        healthcheck = self.create_healthcheck()
        self.elb.configure_health_check(name, healthcheck)

        if instances:
            self.register_instances(name, instances)

        ### Validate the creation of the load balancer
        lbs = self.elb.get_all_load_balancers(load_balancer_names=[name])
        if not "load_balancers" in self.test_resources:
            self.test_resources["load_balancers"] = []

        if len(lbs) == 1:
            self.test_resources["load_balancers"].append(lbs[0])
            return lbs[0]
        else:
            raise Exception("Unable to retrieve load balancer after creation")

    def delete_load_balancers(self, lbs, timeout=60):
        for lb in lbs:
            self.delete_load_balancer(lb, timeout)

    def delete_load_balancer(self, lb, timeout=60, poll_sleep=10):
        self.debug("Deleting Loadbalancer: {0}".format(lb.name))
        self.elb.delete_load_balancer(lb.name)
        poll_count = timeout / poll_sleep
        for _ in range(poll_count):
            lbs = self.elb.get_all_load_balancers(load_balancer_names=[lb.name])
            if lb in lbs:
                time.sleep(poll_sleep)
        if lb in self.test_resources["load_balancers"]:
            self.test_resources["load_balancers"].remove(lb)

    def create_app_cookie_stickiness_policy(self, name, lb_name, policy_name):
        self.debug("Create app cookie stickiness policy: " + str(policy_name))
        self.elb.create_app_cookie_stickiness_policy(name=name,
                                                     lb_name=lb_name,
                                                     policy_name=policy_name)

    def create_lb_cookie_stickiness_policy(self, cookie_expiration_period, lb_name, policy_name):
        self.debug("Create lb cookie stickiness policy: " + str(policy_name))
        self.elb.create_lb_cookie_stickiness_policy(cookie_expiration_period=cookie_expiration_period,
                                                    lb_name=lb_name,
                                                    policy_name=policy_name)

    def create_lb_policy(self, lb_name, policy_name, policy_type, policy_attributes):
        self.debug("Create lb policy: " + str(policy_name))
        self.elb.create_lb_policy(lb_name=lb_name,
                                  policy_name=policy_name,
                                  policy_type=policy_type,
                                  policy_attributes=policy_attributes)

    def set_lb_policy(self, lb_name, lb_port, policy_name=None):
        self.debug("Set policy " + str(policy_name) + " for " + lb_name)
        self.elb.set_lb_policies_of_listener(lb_name=lb_name,
                                             lb_port=lb_port,
                                             policies=policy_name)

    def delete_lb_policy(self, lb_name, policy_name):
        self.debug("Deleting lb policy " + str(policy_name) + " from " + str(lb_name))
        self.elb.delete_lb_policy(lb_name=lb_name,
                                  policy_name=policy_name)

    def describe_lb_policies(self, lb):
        lbs = self.elb.get_all_load_balancers(load_balancer_names=[lb])
        return lbs[0].policies

    def add_lb_listener(self, lb_name, listener):
        self.debug("adding listener")
        self.elb.create_load_balancer_listeners(name=lb_name, listeners=[listener])

    def remove_lb_listener(self, lb_name, port):
        self.debug("removing listener")
        self.elb.delete_load_balancer_listeners(name=lb_name, ports=[port])

    def add_server_cert(self, cert_name, cert_dir="./test_data", cert_file="ssl_server_certs_basics.crt",
                           key_file="ssl_server_certs_basics.pem"):
        cert_body = open(join(cert_dir, cert_file)).read()
        cert_key = open(join(cert_dir, key_file)).read()
        self.upload_server_cert(cert_name=cert_name, cert_body=cert_body, private_key=cert_key)


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
from eutester.eutestcase import EutesterTestCase
from testcases.cloud_user.images.imageutils import ImageUtils

machine=None
testcase = EutesterTestCase()

testcase.setup_parser(testname='load_hvm_image.py',
                      description='Loads an hvm image from either a remote url or local file path',
                      emi=False,
                      testlist=False)

testcase.parser.add_argument('--url',help='URL containing remote windows image '
                                          'to create EMI from', default=None)
testcase.parser.add_argument('--filepath',dest='filepath', help='File path to create EMI from', default=None)
testcase.parser.add_argument('--workerip',dest='worker_machine', help='The ip/hostname of the machine that the operation will be performed on', default=None)
testcase.parser.add_argument('--worker_username',dest='worker_username', help='The username of the machine that the operation will be performed on, default:"root"', default='root')
testcase.parser.add_argument('--worker_password',dest='worker_password', help='The password of the machine that the operation will be performed on', default=None)
testcase.parser.add_argument('--worker_keypath',dest='worker_keypath', help='The ssh keypath of the machine that the operation will be performed on', default=None)
testcase.parser.add_argument('--destpath',help='The path on the workip, that this operation will be performed on', default='/disk1/storage')
testcase.parser.add_argument('--urlpass', dest='wget_password',help='Password needed to retrieve remote url', default=None)
testcase.parser.add_argument('--urluser',dest='wget_user', help='Username needed to retrieve remote url', default=None)
testcase.parser.add_argument('--gigtime',dest='time_per_gig', help='Time allowed per gig size of image to be used', default=300)
testcase.parser.add_argument('--interbundletime',dest='inter_bundle_timeout', help='Inter-bundle timeout', default=120)
testcase.parser.add_argument('--virtualization_type', help='virtualization type, hvm or pv', default='hvm')
testcase.parser.add_argument('--bucket',dest='bucketname', help='bucketname', default=None)
testcase.parser.add_argument('--platform', dest='platform', help='"Linux" or "Windows", default: "linux"' , default=None)
testcase.parser.add_argument('--uploaded_manifest', dest='upload_manifest', help='bucket/prefix location of manifest to register' , default=None)
testcase.parser.add_argument('--bundle_manifest', dest='bundle_manifest', help='file path on worker to bundle manifest to upload' , default=None)
testcase.parser.add_argument('--overwrite', help='Will overwrite files in matching work dir on worker machine if found', action='store_true', default=False)

testcase.parser.add_argument('--time_per_gig', help='Time allowed per image size in GB before timing out. Default:300 seconds', default=300)

testcase.get_args()

testcase.args.worker_password = testcase.args.worker_password or testcase.args.password
testcase.args.worker_keypath = testcase.args.worker_keypath or testcase.args.keypair

if not testcase.args.upload_manifest and not testcase.args.bundle_manifest:
    if (not testcase.args.url and not testcase.args.filepath) or (testcase.args.url and testcase.args.filepath):
        raise Exception('If manifest not provieded, either a URL or FILE path is required to create image ')

def make_image_public():
    emi = image_utils.tester.test_resources['images'][0]
    emi.set_launch_permissions(group_names=['all'])
    testcase.debug('\n---------------------------\nCreated EMI:' + str(emi) +'\n---------------------------')


#Create an ImageUtils helper from the arguments provided in this testcase...
image_utils = testcase.do_with_args(ImageUtils)

#Create a single testcase to wrap and run the EMI creation task. Note by default all the overlapping args from
# this testcase are fed to the testunit method when ran.
test1 = testcase.create_testunit_from_method(image_utils.create_emi)
test2 = testcase.create_testunit_from_method(make_image_public)

result = testcase.run_test_case_list([test1, test2], eof=True, clean_on_exit=False, printresults=True)

#By default created resources are stored in the eucaops/tester object's test_resources dict. See if our image is
#prsent. If so print it out...
if image_utils.tester.test_resources['images']:
    emi = image_utils.tester.test_resources['images'][0]
    testcase.debug('\n---------------------------\nCreated EMI:' + str(emi) +'\n---------------------------')

exit(result)



import json

__author__ = 'viglesias'
import re
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class Install(EutesterTestCase):

    def __init__(self, download_creds=False, extra_args= None):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--euca-url",)
        self.parser.add_argument("--enterprise-url")
        self.parser.add_argument("--branch")
        self.parser.add_argument("--nogpg",action='store_true')
        self.parser.add_argument("--nightly",action='store_true')
        self.parser.add_argument("--lvm-extents")
        self.parser.add_argument("--vnet-mode", default="MANAGED-NOVLAN")
        self.parser.add_argument("--vnet-subnet", default="1.0.0.0")
        self.parser.add_argument("--vnet-netmask", default="255.255.0.0")
        self.parser.add_argument("--vnet-publicips")
        self.parser.add_argument("--vnet-privateips")
        self.parser.add_argument("--vnet_addrspernet", default="32")
        self.parser.add_argument("--vnet_privinterface", default="br0")
        self.parser.add_argument("--vnet_pubinterface", default="br0")
        self.parser.add_argument("--vnet_bridge", default="br0")
        self.parser.add_argument("--vnet-dns", default="8.8.8.8")
        self.parser.add_argument("--root-lv", default="/dev/vg01/")
        self.parser.add_argument("--dnsdomain")
        self.parser.add_argument("--block-device-manager", default="das")
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password, download_creds=download_creds)
        #if not self.args.branch and not self.args.euca_url and not self.args.enterprise_url:
        #    self.args.branch = self.args.upgrade_to_branch

    def clean_method(self):
        pass

    def get_repo_url(self, repo = "eucalyptus", branch = "testing"):
        import httplib
        api_host = "packages.release.eucalyptus-systems.com"
        machine = self.tester.get_component_machines("clc")[0]
        path="/api/1/genrepo/?distro="+str(machine.distro.name)+"&releasever=6&arch=x86_64&url=repo-euca@git.eucalyptus-systems.com:"+str(repo)+"&ref="+str(branch) + "&allow-old"
        conn=httplib.HTTPConnection(api_host)
        conn.request("GET", path)
        res=conn.getresponse()
        repo_url = res.read().strip()
        self.tester.debug("Setting " + repo + " URL to: " + repo_url)
        return repo_url

    def add_euca_repo(self):
        if self.args.branch:
            self.args.euca_url = self.get_repo_url("eucalyptus", self.args.branch)
        for machine in self.tester.config["machines"]:
            if machine.distro.name is "vmware":
                continue
            machine.add_repo(self.args.euca_url,"euca-upgrade")

    def add_enterprise_repo(self):
        if self.args.branch:
            self.args.enterprise_url =self.get_repo_url("internal", self.args.branch)
        for machine in self.tester.config["machines"]:
            if machine.distro.name is "vmware":
                continue
            machine.add_repo(self.args.enterprise_url, "ent-upgrade")

    def add_repos(self):
        for machine in self.tester.get_component_machines("clc"):
            machine.package_manager.add_repo(self.repo_url, "eucalyptus")
            machine.package_manager.add_repo(self.euca2ools_url, "euca2ools")
        for machine in self.tester.get_component_machines("ws"):
            machine.package_manager.add_repo(self.repo_url, "eucalyptus")
        for machine in self.tester.get_component_machines("cc"):
            machine.package_manager.add_repo(self.repo_url, "eucalyptus")
        for machine in self.tester.get_component_machines("sc"):
            machine.package_manager.add_repo(self.repo_url, "eucalyptus")
        for machine in self.tester.get_component_machines("nc"):
            if machine.distro.name is "vmware":
                continue
            machine.package_manager.add_repo(self.repo_url, "eucalyptus")
            machine.package_manager.add_repo(self.euca2ools_url, "euca2ools")

    def install_packages(self):
        for machine in self.tester.config["machines"]:
            machine.package_manager.install("ntp")
        for machine in self.tester.get_component_machines("clc"):
            machine.package_manager.install("eucalyptus-cloud")
        for machine in self.tester.get_component_machines("ws"):
            machine.package_manager.install("eucalyptus-walrus")
        for machine in self.tester.get_component_machines("cc"):
            machine.package_manager.install("eucalyptus-cc")
        for machine in self.tester.get_component_machines("sc"):
            machine.package_manager.install("eucalyptus-sc")
        for machine in self.tester.get_component_machines("nc"):
            if machine.distro.name is "vmware":
                continue
            machine.package_manager.install("eucalyptus-nc")

    def start_components(self):
        for machine in self.tester.get_component_machines("nc"):
            if machine.distro.name is "vmware":
                continue
            machine.sys("service eucalyptus-nc start", timeout=480)
        for machine in self.tester.get_component_machines("cc"):
            machine.sys("service eucalyptus-cc start", timeout=480)
        for machine in self.tester.get_component_machines("sc"):
            machine.sys("service eucalyptus-cloud start", timeout=480)
        for machine in self.tester.get_component_machines("ws"):
            machine.sys("service eucalyptus-cloud start", timeout=480)
        for machine in self.tester.get_component_machines("clc"):
            machine.sys("service eucalyptus-cloud start", timeout=480)

    def stop_components(self):
        for machine in self.tester.get_component_machines("clc"):
            machine.sys("service eucalyptus-cloud stop", timeout=480)
        for machine in self.tester.get_component_machines("sc"):
            machine.sys("service eucalyptus-cloud stop", timeout=480)
        for machine in self.tester.get_component_machines("ws"):
            machine.sys("service eucalyptus-cloud stop", timeout=480)
        for machine in self.tester.get_component_machines("cc"):
            machine.sys("service eucalyptus-cc stop", timeout=480)
        for machine in self.tester.get_component_machines("nc"):
            if machine.distro.name is "vmware":
                continue
            machine.sys("service eucalyptus-nc stop", timeout=480)

    def initialize_db(self):
        first_clc = self.tester.get_component_machines("clc")[0]
        first_clc.sys("euca_conf --initialize")

    def setup_bridges(self):
        nc_machines = self.tester.get_component_machines("nc")
        bridge_interface = "em1"
        for nc in nc_machines:
            if nc.distro.name is "vmware":
                return
            nc.sys("echo 'DEVICE=br0\nBOOTPROTO=dhcp\nONBOOT=yes\nTYPE=Bridge' > /etc/sysconfig/network-scripts/ifcfg-br0")
            nc.sys("echo 'DEVICE=" + bridge_interface +"\nTYPE=Ethernet\nBRIDGE=br0' > /etc/sysconfig/network-scripts/ifcfg-" + bridge_interface)
            nc.sys("service network restart")

    def extend_logical_volume(self, logical_volume="/dev/vg01/lv_root", extents="50%FREE"):
        if self.args.root_lv:
            logical_volume = self.args.root_lv
        if self.args.lv_extents:
            logical_volume= self.args.lv_extents
        for machine in self.tester.config["machines"]:
            machine.sys("lvextend " + logical_volume + " -l" + extents )
            machine.sys("resize2fs -f " + logical_volume, timeout=12000)

    def wait_for_creds(self, timeout=900):
        def get_creds():
            try:
                self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password,download_creds=False)
                self.tester.get_credentials()
                return True
            except Exception:
                return False
        self.tester.wait_for_result(get_creds, True, timeout=timeout)

    def sync_ssh_keys(self):
        ### Sync CLC SSH key to all machines
        clc = self.tester.get_component_machines("clc")[0]
        clc_pub_key = clc.sys("cat ~/.ssh/id_rsa.pub")[0]
        for machine in self.tester.get_component_machines():
            machine.sys("echo " + clc_pub_key + " >> ~/.ssh/authorized_keys")

        ### Sync CC keys to the proper NCs
        try:
            for cluster in ["00", "01", "02", "03"]:
                for machine in self.tester.get_component_machines("cc" + cluster):
                    cc_pub_key = machine.sys("cat ~/.ssh/id_rsa.pub")[0]
                    for nc in self.tester.get_component_machines("nc" + cluster):
                        nc.sys("echo " + cc_pub_key + " >> ~/.ssh/authorized_keys")
        except IndexError:
            pass

    def remove_host_check(self):
        for machine in self.tester.get_component_machines():
            ssh_config_file = 'Host *\nStrictHostKeyChecking no\nUserKnownHostsFile=/dev/null\n'
            #assert isinstance(machine, Machine)
            ssh_config_file = machine.sftp.open("/root/.ssh/config", "w")
            ssh_config_file.write(ssh_config_file)
            ssh_config_file.close()

    def register_components(self):
        clcs = self.tester.get_component_machines("clc")
        if len(clcs) > 1:
            clcs[0].sys("euca_conf --register-cloud -C eucalyptus -P eucalyptus -H " + clcs[1].hostname)
        walrii = self.tester.get_component_machines("ws")
        for walrus in walrii:
            clcs[0].sys("euca_conf --register-walrus -C walrus -P walrus -H " + walrus.hostname)
        ccs = self.tester.get_component_machines("cc")
        registered_clusters = {1:None, 2:None,3:None,4:None,5:None,6:None,7:None,8:None}
        cluster_number = 1
        for cluster in ccs:
            if not registered_clusters[cluster_number]:
                registered_clusters[cluster_number] = [cluster]
                clcs[0].sys("euca_conf --register-cluster -C cluster" + str(cluster_number) +
                            "A -P cluster" + str(cluster_number) + " -H " + cluster.hostname)
                cluster_number += 1
            else:
                registered_clusters[cluster_number-1].append(cluster)
                clcs[0].sys("euca_conf --register-cluster -C cluster" + str(cluster_number) +
                            "B -P cluster" + str(cluster_number) + " -H " + cluster.hostname)

        cluster_number = 1
        scs = self.tester.get_component_machines("sc")
        registered_scs = {1:None, 2:None,3:None,4:None,5:None,6:None,7:None,8:None}
        for sc in scs:
            if not registered_scs[cluster_number]:
                registered_scs[cluster_number] = [sc]
                clcs[0].sys("euca_conf --register-sc -C storage" + str(cluster_number) +
                            "A -P cluster" + str(cluster_number) + " -H " + sc.hostname)
                cluster_number += 1
            else:
                registered_scs[cluster_number-1].append(sc)
                clcs[0].sys("euca_conf --register-sc -C storage" + str(cluster_number-1) +
                            "B -P cluster" + str(cluster_number-1) + " -H " + sc.hostname)

        try:
            for cluster in ["00", "01", "02","03"]:
                for nc in self.tester.get_component_machines("nc" + cluster):
                    for cc in self.tester.get_component_machines("cc" + cluster):
                        cc.sys("euca_conf --register-nodes " + nc.hostname)
        except IndexError:
            pass

    def set_block_storage_manager(self):
        if not hasattr(self.tester, 'service_manager'):
            self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
        enabled_clc = self.tester.service_manager.get_enabled_clc()
        self.zones = self.tester.get_zones()
        for zone in self.zones:
            ebs_manager = "overlay"
            if hasattr(self.args, 'ebs_storage_manager'):
                if re.search("DASManager" ,self.args.ebs_storage_manager):
                    ebs_manager = "das"
                if re.search("SANManager" ,self.args.ebs_storage_manager):
                    if hasattr(self.args, 'san_provider'):
                        if re.search("EquallogicProvider", self.args.san_provider):
                            ebs_manager = "equallogic"
                        if re.search("NetappProvider", self.args.san_provider):
                            ebs_manager = "netapp"
                        if re.search("EmcVnxProvider", self.args.san_provider):
                            ebs_manager = "emc-fastsnap"
            else:
                ebs_manager = self.args.block_device_manager
            self.tester.modify_property(zone + ".storage.blockstoragemanager", ebs_manager)
            self.tester.modify_property(zone + ".storage.dasdevice", self.args.root_lv)

    def set_config_option(self, machine, option, parameter):
        sed_command = 'sed -i -e "s/^.*{0}=.*$/{0}={1}/" {2}/etc/eucalyptus/eucalyptus.conf'.format(option, parameter, self.tester.eucapath)
        machine.sys(sed_command)

    def configure_network(self):
        for machine in self.tester.get_component_machines("cc"):
            self.set_config_option(machine, "VNET_MODE", self.args.vnet_mode)
            self.set_config_option(machine, "VNET_SUBNET", self.args.vnet_subnet)
            self.set_config_option(machine, "VNET_NETMASK", self.args.vnet_netmask)
            self.set_config_option(machine, "VNET_PUBLICIPS", self.args.vnet_publicips)
            self.set_config_option(machine, "VNET_DNS", self.args.vnet_dns)
            self.set_config_option(machine, "VNET_ADDRSPERNET", self.args.vnet_addrspernet)
            self.set_config_option(machine, "VNET_PRIVINTERFACE", self.args.vnet_privinterface)
            self.set_config_option(machine, "VNET_PUBINTERFACE", self.args.vnet_pubinterface)

        for machine in self.tester.get_component_machines("nc"):
            self.set_config_option(machine, "VNET_MODE", self.args.vnet_mode)
            self.set_config_option(machine, "VNET_BRIDGE", self.args.vnet_bridge)

    def configure_edge_dual_subnet(self):
        if re.search("edge", self.tester.config["network"], re.IGNORECASE):
            if not hasattr(self.tester, 'service_manager'):
                self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
            enabled_clc = self.tester.service_manager.get_enabled_clc()
            ip_addr_add_command = "ip addr add {0}/24 dev {1}"
            ### add private subnet GW interface on CC
            cc = self.tester.get_component_machines("cc00")[0]
            cc_interface = "em1"
            base_ip = self.tester.config["subnet_ip"].strip("0")
            private_subnet = base_ip + "0"
            cc_gw_ip = base_ip + "1"
            cc.sys(ip_addr_add_command.format(cc_gw_ip, cc_interface))

            ### Setup IP forwarding
            cc.sys("sysctl -w net.ipv4.ip_forward=1")

            ### add private interface subnet on NC bridges
            ip_index = 2
            for nc in self.tester.get_component_machines("nc"):
                nc_private_ip = base_ip + str(ip_index)
                ip_index += 1
                nc.sys(ip_addr_add_command.format(nc_private_ip, "br0"))

            ### Chunk out private ip pool
            instance_private_ips = []
            for i in xrange(64):
                instance_private_ips.append(base_ip + str(ip_index + i))
            if hasattr(self.args, "vnet_public_ips"):
                public_ips = self.args.vnet_public_ips
            elif "managed_ips" in self.tester.config:
                public_ips = self.tester.config["managed_ips"]
            else:
                raise Exception("Unable to find public ips. Please provide them with --vnet-public-ips")
            ip_list = public_ips.split()
            if len(ip_list) > 1:
                ### individual ips were passed
                public_ips = ip_list
            else:
                public_ips = [public_ips]

            network_config = {"InstanceDnsDomain": "eucalyptus.internal",
                              "InstanceDnsServers": [enabled_clc.hostname],
                              "PublicIps": public_ips,
                              "Subnets": [],
                              "Clusters": [{"Name": "PARTI00",
                                            "MacPrefix": "d0:0d",
                                            "Subnet": {
                                                "Name": private_subnet,
                                                "Subnet": private_subnet,
                                                "Netmask": "255.255.255.0",
                                                "Gateway": cc_gw_ip
                                            },
                                            "PrivateIps": instance_private_ips
                                            }]
                              }
            self.tester.modify_property("cloud.network.network_configuration", "'" + json.dumps(network_config) + "'")

    def setup_dns(self):
        if not hasattr(self.tester, 'service_manager'):
            self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
        self.tester.modify_property("bootstrap.webservices.use_dns_delegation", "true")
        self.tester.modify_property("bootstrap.webservices.use_instance_dns", "true")
        enabled_clc = self.tester.service_manager.get_enabled_clc()
        if self.args.dnsdomain:
            self.tester.modify_property("system.dns.dnsdomain", self.args.dnsdomain)
        else:
            hostname = enabled_clc.machine.sys('hostname')[0].split(".")[0]
            domain = hostname + ".autoqa.qa1.eucalyptus-systems.com"
            self.tester.modify_property("system.dns.dnsdomain", domain)
        self.tester.modify_property("system.dns.nameserveraddress", enabled_clc.hostname)

    def clean_method(self):
        pass

    def InstallEuca(self):
        self.initialize_db()
        self.sync_ssh_keys()
        self.remove_host_check()
        self.configure_network()
        self.start_components()
        self.wait_for_creds()
        self.register_components()
        self.set_block_storage_manager()

if __name__ == "__main__":
    testcase = Install()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    test_list = testcase.args.tests or ["InstallEuca"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in test_list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
from eutester import Eutester
import re
import boto

class IAMops(Eutester):

    def __init__(self,credpath=None, endpoint="iam.amazonaws.com", aws_access_key_id=None, aws_secret_access_key=None,
                 is_secure=True, port=443, path='/', boto_debug=0 ):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.user_id = None
        self.account_id = None
        super(IAMops, self).__init__(credpath=credpath)
        self.setup_iam_connection(endpoint=endpoint, aws_access_key_id=self.aws_access_key_id ,
                                  aws_secret_access_key=self.aws_secret_access_key, is_secure=is_secure, port=port,
                                  path=path, boto_debug=boto_debug )

    def setup_iam_connection(self, endpoint="iam.amazonaws.com", aws_access_key_id=None,aws_secret_access_key=None,
                             is_secure=True, port=443, path='/', boto_debug=0 ):
        try:
            euare_connection_args = { 'aws_access_key_id' : aws_access_key_id,
                                      'aws_secret_access_key': aws_secret_access_key,
                                      'is_secure': is_secure,
                                      'debug':boto_debug,
                                      'port' : port,
                                      'path' : path,
                                      'host' : endpoint}
            self.debug("Attempting to create IAM connection to " + endpoint + ':' + str(port) + path)
            self.euare = boto.connect_iam(**euare_connection_args)
        except Exception, e:
            self.critical("Was unable to create IAM connection because of exception: " + str(e))

    def get_iam_ip(self):
        """Parse the eucarc for the EUARE_URL"""
        iam_url = self.parse_eucarc("EUARE_URL")
        return iam_url.split("/")[2].split(":")[0]

    def get_iam_path(self):
        """Parse the eucarc for the EUARE_URL"""
        iam_url = self.parse_eucarc("EUARE_URL")
        iam_path = "/".join(iam_url.split("/")[3:])
        return iam_path

    def create_account(self,account_name):
        """
        Create an account with the given name

        :param account_name: str name of account to create
        """
        self.debug("Creating account: " + account_name)
        params = {'AccountName': account_name}
        self.euare.get_response('CreateAccount', params)
    
    def delete_account(self,account_name,recursive=False):
        """
        Delete an account with the given name

        :param account_name: str name of account to delete
        :param recursive:
        """
        self.debug("Deleting account: " + account_name)
        params = {
            'AccountName': account_name,
            'Recursive': recursive
        }
        self.euare.get_response('DeleteAccount', params)

    def get_all_accounts(self, account_id=None, account_name=None, search=False):
        """
        Request all accounts, return account dicts that match given criteria

        :param account_id: regex string - to use for account_name
        :param account_name: regex - to use for account ID
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return: list of account names
        """
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        self.debug('Attempting to fetch all accounts matching- account_id:'+str(account_id)+' account_name:'+str(account_name))
        response = self.euare.get_response('ListAccounts',{}, list_marker='Accounts')
        retlist = []
        for account in response['list_accounts_response']['list_accounts_result']['accounts']:
            if account_name is not None and not re_meth( account_name, account['account_name']):
                continue
            if account_id is not None and not re_meth(account_id, account['account_id']):
                continue
            retlist.append(account)
        return retlist
             
    def create_user(self, user_name,path="/", delegate_account=None):
        """
        Create a user

        :param user_name: str name of user
        :param path: str user path
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Attempting to create user: " + user_name)
        params = {'UserName': user_name,
                  'Path': path }
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('CreateUser',params)
    
    def delete_user(self, user_name, delegate_account=None):
        """
        Delete a user

        :param user_name: str name of user
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Deleting user " + user_name)
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('DeleteUser', params)

    def get_users_from_account(self, path=None, user_name=None, user_id=None, delegate_account=None, search=False):
        """
        Returns users that match given criteria. By default will return current account.

        :param path: regex - to match for path
        :param user_name: str name of user
        :param user_id: regex - to match for user_id
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        :param search: use regex search (any occurrence) rather than match (exact same strings must occur)
        :return:
        """
        self.debug('Attempting to fetch all users matching- user_id:'+str(user_id)+' user_name:'+str(user_name)+" acct_name:"+str(delegate_account))
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        if delegate_account:
            params['DelegateAccount'] = delegate_account         
        response = self.euare.get_response('ListUsers', params, list_marker='Users')
        for user in response['list_users_response']['list_users_result']['users']:
            if path is not None and not re_meth(path, user['path']):
                continue
            if user_name is not None and not re_meth(user_name, user['user_name']):
                continue
            if user_id is not None and not re_meth(user_id, user['user_id']):
                continue
            retlist.append(user)
        return retlist

    def show_all_accounts(self, account_name=None, account_id=None, search=False ):
        """
        Debug Method to print an account list based on given filter criteria

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for account_id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        list = self.get_all_accounts(account_name=account_name, account_id=account_id, search=search)
        self.debug('-----------------------------------------------------------------------')
        self.debug(str('ACCOUNT_NAME:').ljust(25) + "|" + str('ACCT_ID:'))
        self.debug('-----------------------------------------------------------------------')
        for account in list:
            self.debug(str(account['account_name']).ljust(25) + "|" +str(account['account_id']))
            self.debug('-----------------------------------------------------------------------')

    def show_all_groups(self, account_name=None,  account_id=None,  path=None, group_name=None,  group_id=None,  search=False):
        """
        Print all groups in an account

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param group_name: regex - to match for user_name
        :param group_id: regex - to match for user_id
        :param search:  boolean - specify whether to use match or search when filtering the returned list
        """
        list = self.get_all_groups(account_name=account_name, account_id=account_id, path=path, group_name=group_name, group_id=group_id, search=search)
        self.debug('-----------------------------------------------------------------------')
        self.debug(str('ACCOUNT:').ljust(25) + "|" + str('GROUPNAME:').ljust(15) + "|" + str('GROUP_ID:').ljust(25)  )
        self.debug('-----------------------------------------------------------------------')
        for group in list:
            self.debug(str(group['account_name']).ljust(25) + "|" + str(group['group_name']).ljust(15) + "|" + str(group['group_id']))
            self.debug('-----------------------------------------------------------------------')

    def show_all_users(self, account_name=None, account_id=None,  path=None, user_name=None,  user_id=None, search=False ):
        """
        Debug Method to print a user list based on given filter criteria

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param user_name: regex - to match for user_name
        :param user_id: regex - to match for user_id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        list = self.get_all_users(account_name=account_name, account_id=account_id, path=path, user_name=user_name, user_id=user_id, search=search)
        self.debug('-----------------------------------------------------------------------')
        self.debug(str('ACCOUNT:').ljust(25) + "|" + str('USERNAME:').ljust(15) + "|" + str('USER_ID').ljust(25) + "|" + str('ACCT_ID') )
        self.debug('-----------------------------------------------------------------------')
        for user in list:
            self.debug(str(user['account_name']).ljust(25) + "|" + str(user['user_name']).ljust(15) + "|" + str(user['user_id']).ljust(25) + "|" + str(user['account_id']))
            self.debug('-----------------------------------------------------------------------')
    def get_euare_username(self):
        """
        Get all users in the current users account
        """
        return self.get_all_users(account_id=str(self.get_account_id()))[0]['user_name']
    
    def get_euare_accountname(self):
        """
        Get account name of current user
        """
        return self.get_all_users(account_id=str(self.get_account_id()))[0]['account_name']

    def get_all_users(self,  account_name=None,  account_id=None,  path=None, user_name=None,  user_id=None,  search=False ):
        """
        Queries all accounts matching given account criteria, returns all users found within these accounts which then match the given user criteria.
        Account info is added to the user dicts

        :param account_name: regex - to use for account name
        :param account_id: regex - to use for account id
        :param path: regex - to match for path
        :param user_name: regex - to match for user name
        :param user_id: regex - to match for user id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return: List of users with account name tuples
        """
        userlist=[]
        accounts = self.get_all_accounts(account_id=account_id, account_name=account_name, search=search)
        for account in accounts:
            if account['account_id'] == self.account_id:
                users =self.get_users_from_account()
            else:
                users = self.get_users_from_account(path=path, user_name=user_name, user_id=user_id, delegate_account=account['account_name'], search=search)
            for user in users:
                user['account_name']=account['account_name']
                user['account_id']=account['account_id']
                userlist.append(user)
        return userlist

    def get_user_policy_names(self, user_name, policy_name=None,delegate_account=None, search=False):
        """
        Returns list of policy names associated with a given user, and match given criteria.

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return: list of policy names
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.euare.get_response('ListUserPolicies',params, list_marker='PolicyNames')
        for name in response['list_user_policies_response']['list_user_policies_result']['policy_names']:
            if policy_name is not None and not re_meth(policy_name, name):
                continue
            retlist.append(name)
        return retlist

    def get_user_policies(self, user_name, policy_name=None,delegate_account=None, doc=None, search=False):
        """
        Returns list of policy dicts associated with a given user, and match given criteria.

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return:
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        names = self.get_user_policy_names(user_name, policy_name=policy_name, delegate_account=delegate_account, search=search)
        
        for p_name in names:
            params = {'UserName': user_name,
                      'PolicyName': p_name}
            if delegate_account:
                params['DelegateAccount'] = delegate_account
            policy = self.euare.get_response('GetUserPolicy', params, verb='POST')['get_user_policy_response']['get_user_policy_result']
            if doc is not None and not re_meth(doc, policy['policy_document']):
                continue
            retlist.append(policy)
        return retlist
        
    def show_user_policy_summary(self,user_name,policy_name=None,delegate_account=None, doc=None, search=False):
        """
        Debug method to display policy summary applied to a given user

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        policies = self.get_user_policies(user_name, policy_name=policy_name, delegate_account=delegate_account, doc=doc, search=search)
        for policy in policies:
            self.debug('-------------------------------------')
            self.debug("\tPOLICY NAME: "+str(policy['policy_name']) +", USER_NAME: " +str(user_name))
            self.debug('-------------------------------------')
            for line in str(policy['policy_document']).splitlines():
                self.debug("\t"+line)
    
    def show_user_summary(self,user_name, delegate_account=None, account_id=None):
        """
        Debug method for to display euare/iam info for a specific user.

        :param user_name: string - user to get policies for.
        :param delegate_account: string - used for user lookup
        :param account_id: regex - to use for account id
        """
        user_name = user_name
        if delegate_account is None:
            account_id=self.get_account_id()
            delegate_account= self.get_all_accounts(account_id=account_id)[0]['account_name']
        self.debug('Fetching user summary for: user_name:'+str(user_name)+" account:"+str(delegate_account)+" account_id:"+str(account_id))
        self.show_all_users(account_name=delegate_account, account_id=account_id, user_name=user_name)
        self.show_user_policy_summary(user_name, delegate_account=delegate_account)
        
        
    def show_euare_whoami(self):
        """
        Debug method used to display the who am I info related to iam/euare.
        """
        user= self.euare.get_user()['get_user_response']['get_user_result']['user']
        user_id = user['user_id']
        user_name = user['user_name']
        account_id = self.get_account_id()
        self.show_all_users(account_id=account_id, user_id=user_id)
        self.show_user_policy_summary(user_name)
        
    
    def attach_policy_user(self, user_name, policy_name, policy_json, delegate_account=None):
        """
        Attach a policy string to a user

        :param user_name: string - user to apply policy to
        :param policy_name: Name to upload policy as
        :param policy_json: Policy text
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Attaching the following policy to " + user_name + ":" + policy_json)
        params = {'UserName': user_name,
                  'PolicyName': policy_name,
                  'PolicyDocument': policy_json}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('PutUserPolicy', params, verb='POST')
    
    def detach_policy_user(self, user_name, policy_name, delegate_account=None):
        """
        Detach a policy from user

        :param user_name: string - user to apply policy to
        :param policy_name: Name to upload policy as
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Detaching the following policy from " + user_name + ":" + policy_name)
        params = {'UserName': user_name,
                  'PolicyName': policy_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('DeleteUserPolicy', params, verb='POST')

    def get_all_groups(self, account_name=None, account_id=None, path=None, group_name=None, group_id=None, search=False ):
        """
        Queries all accounts matching given account criteria, returns all groups found within these accounts which then match the given user criteria.
        Account info is added to the group dicts

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param group_name: regex - to match for group_name
        :param group_id: regex - to match for group_id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return:
        """
        grouplist=[]
        accounts = self.get_all_accounts(account_id=account_id, account_name=account_name, search=search)
        for account in accounts:
            groups = self.get_groups_from_account(path=path, group_name=group_name, group_id=group_id, delegate_account=account['account_name'], search=search)
            for group in groups:
                group['account_name']=account['account_name']
                group['account_id']=account['account_id']
                grouplist.append(group)
        return grouplist

    def get_groups_from_account(self, path=None, group_name=None, group_id=None, delegate_account=None, search=False):
        """
        Returns groups that match given criteria. By default will return groups from current account.

        :param path: regex - to match for path
        :param group_name: regex - to match for group_name
        :param group_id: regex - to match for group_id
        :param delegate_account: string - to use for delegating account lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return:
        """
        self.debug('Attempting to fetch all groups matching- group_id:'+str(group_id)+' group_name:'+str(group_name)+" acct_name:"+str(delegate_account))
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        if delegate_account:
            params['DelegateAccount'] = delegate_account         
        response = self.euare.get_response('ListGroups', params, list_marker='Groups')
        for group in response['list_groups_response']['list_groups_result']['groups']:
            if path is not None and not re_meth(path, group['path']):
                continue
            if group_name is not None and not re_meth(group_name, group['group_name']):
                continue
            if group_id is not None and not re_meth(group_id, group['group_id']):
                continue
            retlist.append(group)
        return retlist
        
    def get_group_policy_names(self, group_name, policy_name=None,delegate_account=None, search=False):
        """
        Returns list of policy names associated with a given group, and match given criteria.

        :param group_name: string - group to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for group lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return: list of policy names
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        params = {'GroupName': group_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.euare.get_response('ListGroupPolicies',params, list_marker='PolicyNames')
        for name in response['list_group_policies_response']['list_group_policies_result']['policy_names']:
            if policy_name is not None and not re_meth(policy_name, name):
                continue
            retlist.append(name)
        return retlist

    def get_group_policies(self, group_name, policy_name=None,delegate_account=None, doc=None, search=False):
        """
        Returns list of policy dicts associated with a given group, and match given criteria.

        :param group_name: string - group to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for group lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return:
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        names = self.get_group_policy_names(group_name, policy_name=policy_name, delegate_account=delegate_account, search=search)

        for p_name in names:
            params = {'GroupName': group_name,
                      'PolicyName': p_name}
            if delegate_account:
                params['DelegateAccount'] = delegate_account
            policy = self.euare.get_response('GetGroupPolicy', params, verb='POST')['get_group_policy_response']['get_group_policy_result']
            if doc is not None and not re_meth(doc, policy['policy_document']):
                continue
            retlist.append(policy)
        return retlist
    
    def create_group(self, group_name,path="/", delegate_account=None):
        """
        Create group.

        :param
        :param path: path for group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Attempting to create group: " + group_name)
        params = {'GroupName': group_name,
                  'Path': path}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('CreateGroup', params)
    
    def delete_group(self, group_name, delegate_account=None):
        """
        Delete group.

        :param group_name: name of group to delete
        :param delegate_account:
        """
        self.debug("Deleting group " + group_name)
        params = {'GroupName': group_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('DeleteGroup', params)
    
    def add_user_to_group(self, group_name, user_name, delegate_account=None):
        """
        Add a user to a group.

        :param group_name: name of group to add user to
        :param user_name: name of user to add to group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Adding user "  +  user_name + " to group " + group_name)
        params = {'GroupName': group_name,
                  'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('AddUserToGroup', params)
    
    def remove_user_from_group(self, group_name, user_name, delegate_account=None):
        """
        Remove a user from a group.

        :param group_name: name of group to remove user from
        :param user_name: name of user to remove from group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Removing user "  +  user_name + " to group " + group_name)
        params = {'GroupName': group_name,
                  'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('RemoveUserFromGroup', params)
    
    def attach_policy_group(self, group_name, policy_name, policy_json, delegate_account=None):
        """
        Attach a policy to a group.

        :param group_name: name of group to remove user from
        :param policy_name: Name to upload policy as
        :param policy_json: Policy text
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Attaching the following policy to " + group_name + ":" + policy_json)
        params = {'GroupName': group_name,
                  'PolicyName': policy_name,
                  'PolicyDocument': policy_json}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('PutGroupPolicy', params, verb='POST')
    
    def detach_policy_group(self, group_name, policy_name, delegate_account=None):
        """
        Remove a policy from a group.

        :param group_name: name of group to remove user from
        :param policy_name: Name to upload policy as
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.debug("Detaching the following policy from " + group_name + ":" + policy_name)
        params = {'GroupName': group_name,
                  'PolicyName': policy_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('DeleteGroupPolicy', params, verb='POST')
    
    def create_access_key(self, user_name=None, delegate_account=None):
        """
        Create a new access key for the user.

        :param user_name: Name of user to create access key for to
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        :return: A tuple of access key and and secret key with keys: 'access_key_id' and 'secret_access_key'
        """
        self.debug("Creating access key for " + user_name )
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.euare.get_response('CreateAccessKey', params)
        access_tuple = {}
        access_tuple['access_key_id'] = response['create_access_key_response']['create_access_key_result']['access_key']['access_key_id']
        access_tuple['secret_access_key'] = response['create_access_key_response']['create_access_key_result']['access_key']['secret_access_key']
        return access_tuple

    def upload_server_cert(self, cert_name, cert_body, private_key):
        self.debug("uploading server certificate: " + cert_name)
        self.euare.upload_server_cert(cert_name=cert_name, cert_body=cert_body, private_key=private_key)
        if cert_name not in str(self.euare.get_server_certificate(cert_name)):
            raise Exception("certificate " + cert_name + " not uploaded")

    def update_server_cert(self, cert_name, new_cert_name=None, new_path=None):
        self.debug("updating server certificate: " + cert_name)
        self.euare.update_server_cert(cert_name=cert_name, new_cert_name=new_cert_name, new_path=new_path)
        if (new_cert_name and new_path) not in str(self.euare.get_server_certificate(new_cert_name)):
            raise Exception("certificate " + cert_name + " not updated.")

    def get_server_cert(self, cert_name):
        self.debug("getting server certificate: " + cert_name)
        cert = self.euare.get_server_certificate(cert_name=cert_name)
        self.debug(cert)
        return cert

    def delete_server_cert(self, cert_name):
        self.debug("deleting server certificate: " + cert_name)
        self.euare.delete_server_cert(cert_name)
        if (cert_name) in str(self.euare.list_server_certs()):
            raise Exception("certificate " + cert_name + " not deleted.")

    def list_server_certs(self, path_prefix='/', marker=None, max_items=None):
        self.debug("listing server certificates")
        certs = self.euare.list_server_certs(path_prefix=path_prefix, marker=marker, max_items=max_items)
        self.debug(certs)
        return certs

    def create_login_profile(self, user_name, password, delegate_account=None):
        self.debug("Creating login profile for: " + user_name + " with password: " + password)
        params = {'UserName': user_name,
                  'Password': password}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.euare.get_response('CreateLoginProfile', params, verb='POST')


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
__author__ = 'clarkmatthew'


from san_volume_info import  San_Volume_Info
from eutester.sshconnection import SshConnection, SshCbReturn, CommandExitCodeException, CommandTimeoutException
from eutester.eulogger import Eulogger
import re
import string




class San_Client():
    '''
    Basic San Client test interface to be implemented by each SAN client. The intention is to use a common
    interface to verify SAN side operations when resources from that san are used in cloud operations.
    '''

    def __init__(self, host, username, password):
            self.host = host
            self.username = username
            self.password = password
            self.connection = self.connect()
            self.logger = Eulogger(identifier= 'SAN-'+str(host))
            self.debug= self.logger.log.debug

    @classmethod
    def get_san_client_by_type(cls, host, username, password, santype):
        '''
        Returns a SAN client type object by santype.

        param host: string representing the hostname/ip of the SAN
        param username: string username used to access the SAN
        param password: string password used to access the SAN
        param santype: string used to determine what SAN client to create and return. Supported Values: 'netapp'
        returns San_Client obj of type santype
        '''
        if santype == 'netapp':
            return Netapp_Client(host, username, password)
        else:
            raise Exception('Unknown santype provided:' + str(santype))

    def connect(self):
        '''
        Will perform the steps needed to connect/interface with the SAN/San_Client
        '''
        raise Exception('Not Implemented')

    def sys(self, cmd, timeout=60, code=0):
        '''
        Will Executes the command provided against the San_Client interface
        param cmd: string representing the command to be executed against this san San_Client
        param timeout: optional timeout for the command which is executed
        param code: optional integer value used to provide the expected return code
        returns sanclient's output from the executed command
        '''
        raise Exception('Not Implemented')

    def get_san_volume_info_by_id(self, volumeid):
        '''
        Attempt to populate a san_volume_info obj with information about a given volumeid
        param volumeid: string representing the volume's id in which to gather information
        returns san_volume_info
        '''
        raise Exception('Not Implemented')



class netapp_menu():

    def __init__(self, path_string,  san_client, help_string = "", dir_list_raw=None, scan_all=False):
        '''
        Represent a Netapp cli menu context.
        param path_string: the relative path or context within the san client Netapp_Client cli
        param san_client: the Netapp_Client
        param help_string: optional string which can be used as a doc string and help string for this menu context
        param dir_list_raw: optional Netapp_Client list of strings representing the menu context
        '''
        self._path_string = path_string
        self._helpstring = help_string
        self.__doc__ = self._helpstring
        self._san_client = san_client
        self._dir_list_raw = dir_list_raw or self._get_dir_list_raw(scan_all=scan_all)
        self._parse_dir_list()

    def _get_dir_list_raw(self, listformat=True, scan_all=False):
        '''
        Will attempt to get the contents of this menu context and will present the text returned from the san Netapp_Client
        in either a single string buffer or list of lines/strings
        param listformat: boolean when true, the output will be returned as a list of strings
        '''
        ret = [] if listformat else ""
        ret = self._san_client.sys("?")
        if scan_all:
            for letter in string.lowercase:
                if listformat:
                    try:
                        scan_results = self._san_client.sys_send_non_exec(str(letter)+"\t", timeout=2)
                        ret.extend(self._parse_scanned_results(scan_results))
                    except CommandTimeoutException:
                        pass
                else:
                    ret += self._san_client.sys_send_non_exec(str(letter)+"\t", listformat=False,timeout=2)
        return ret

    def _parse_scanned_results(self, scan_results, prompt='::>'):
        #todo lookup each key and better distinguish between command and new menu item, also provide more help info
        keywords = []
        ret_list = []
        for line in scan_results:
            if re.search('Error', line, re.IGNORECASE):
                #There's no commands for this specific menu context
                return []
            if line and not re.search(prompt, line):
                split = line.strip().split()
                keywords.extend(split)
        for key in keywords:
            self._san_client.debug('Scan found KEY:'+str(key))
            out = self._san_client.sys(key + "?")
            print "\n".join(str(x) for x in out)
            ret_list.extend(out)
        return ret_list

    def print_help(self):
        '''
        method that will print the help contents for this menu context
        '''
        return self._san_client.debug("\n"+ self._get_dir_list_raw(listformat=False))

    def _exec_sys(self, cmd, verbose=True):
        '''
        Helper function to execute commands on the remote san via the provided san client interface
        param cmd: string representing the command to execute on the san_volume_info
        param verbose: boolean if true will print the command to the san client's debug method
        '''
        if verbose:
            self._san_client.debug(str(cmd))
        return self._san_client.sys(cmd)

    def _get_new_command(self, path, docstring =""):
        '''
        Attempt to create dynamic methods for the menu items found when traversing the cli menu(s)
        param path: string representing the cli menu path to this obj/command
        param docstring: optional string will be used to populate the docstring of the returned method
        '''
        def new_command( string = "" ):
            return self._exec_sys(path + ' ' + string)
        new_command.__doc__ = docstring
        new_command.help = lambda x='?':new_command(x)
        return new_command


    def _parse_dir_list(self):
        '''
        Attempt to traverse and discover  all the items in the menu item and either create executable methods or
        new netapp_menu items which contain more cli methods/menus.
        '''
        self._san_client.debug('Populating CLI dir:' + self._path_string)
        last_obj = None
        for line in self._dir_list_raw:
            split = line.split()
            if split:
                keyname = split.pop(0)
                value = str(" ".join(str(x) for x in split))
                #is this line a help string continuation
                if re.search(',',keyname):
                    if last_obj:
                        last_obj.__doc__ += keyname + value
                #is this line a new menu option
                elif re.search('>', keyname) or re.search("Manage", value):
                    keyname = keyname.replace('>','')
                    dir = netapp_menu(self._path_string + " " + keyname, self._san_client)
                    dir.__doc__ = value
                    setattr(self, keyname, dir)
                    last_obj = dir
                #is this line a command in this menu context
                else:
                    match = re.search('\w+', keyname)
                    if match:
                        keyname = match.group()
                        newpath = str(self._path_string) + " " + str(keyname)
                        docstring = "Command:" + str(newpath) + ". " + value
                        command = self._get_new_command(newpath, docstring=docstring)
                        setattr(self, keyname, command)
                        last_obj = command


class Netapp_Client(San_Client):


    def connect(self):
        '''
        Attempts to connect to the SAN cli via ssh
        '''
        ssh = SshConnection(host=self.host, username=self.username, password=self.password)
        if not ssh:
            raise Exception('Could not connect to SAN at:' + str(self.host))
        self.sys = ssh.sys
        return ssh

    def get_san_volume_info_by_id(self, volumeid):
        '''
        Attempts to gather info from the san client about the volume id provided.
        Returns a San_Volume_Info object representing the volume's san client information

        param volumeid: string representing the volumeid ie:vol-abcd123
        returns San_Volume_Info object
        '''
        info_dict = self._get_volume_basic_info_by_id(volumeid)
        eff_dict = self._get_volume_efficiency_info_by_id(volumeid)
        san_info_obj = San_Volume_Info( volumeid, dict(info_dict.items() + eff_dict.items()), self)
        return san_info_obj

    def _format_volume_id(self,volumeid):
        id = str(volumeid).replace('-','_')
        return id

    def _raw_cb(self, buf, promptcount, promptstring, promptmax, debug):
        '''
        Call back which can be used to control and ssh command when a shell and raw
         input needed (ie to send controlled tabs for example)
        param buf: buffer provided by ssh.cmd session
        param promptcount: int amount of times we've seen the prompt
        param promptstring: string the string to match
        param promptmax: the maximum number prompt matches before the command is signaled 'done'
        returns SshCbReturn obj
        '''
        ret = SshCbReturn(buf=buf, stop=False)
        #Count the number of occurrences of the promptstring
        promptcount = promptcount + len(re.findall(promptstring,buf))
        if promptcount >= promptmax:
            ret.stop = True
        #Return the args this call back will be called with the next time around
        ret.nextargs=[promptcount, promptstring, promptmax, debug]
        if debug:
            print str(buf)+ '\npromptcount:' + str(promptcount) + ', promptstring:' + str(promptstring) + ', promptmax:'+ str(promptmax)
        return ret

    def sys_send_non_exec(self, cmd, promptstring='::>', promptmax=2, listformat=True, timeout=120, debug=False, code=0):
        '''
        Ssh connection command which can be used when raw input to a remote shell
         are needed (ie to send controlled tabs for example)

        param cmd: String to be send to remote ssh shell
        param promptstring: string used to match as the 'prompt'
        param promptmax: the maximum number prompt matches before the command is signaled 'done'
        param code: integer representing the expected return/exit code from the command
        returns
        '''
        out = self.connection.cmd(cmd,
                                  get_pty=True,
                                  invoke_shell=True,
                                  timeout=timeout,
                                  listformat=listformat,
                                  cb=self._raw_cb,
                                  cbargs=[0,promptstring, promptmax,debug])
        output = out['output']
        if code is not None:
            if out['status'] != -1 and out['status'] != code:
                self.debug(output)
                raise CommandExitCodeException('Cmd:' + str(cmd) + ' failed with status code:'
                                               + str(out['status']) + ", output:" + str(output))
        return output

    def _get_volume_basic_info_by_id(self, volumeid):
        cmd = 'volume show ' + str(self._format_volume_id(volumeid))
        return self.get_cmd_dict(cmd, code=0)

    def get_efficiency_policy(self, policy_name):
        pdict = self.get_cmd_dict('volume efficiency policy show ' + str(policy_name))
        if 'efficiency_policy_name' in pdict:
            if pdict['efficiency_policy_name'] == policy_name:
                return pdict
        return None

    #def create_efficiency_policy(self, policy_name, schedule_name, duraction_hours, enabled=True, comment='eutester_created' ):

    def discover_cli_menu(self):
        '''
        Attempts to traverse and discover the CLI menu creating executable python methods and help strings for the
        items found in each menu context. CLI menu items will be available from self.cli
        '''
        self.cli = netapp_menu("", self, help_string='Parent dir for netapp commands', scan_all=True)


    def _get_volume_efficiency_info_by_id(self, volumeid):
        cmd = 'volume efficiency show -volume ' + str(self._format_volume_id(volumeid))
        return self.get_cmd_dict(cmd)


    def get_cmd_dict(self, cmd, code=0):
        '''
        Execute a command via the client and attempt to parse the results into a returned dict
        param cmd: string representing the command to be run on the san client
        returns dict
        '''
        info = {}
        out = self.sys(cmd, code=code)
        for line in out:
            split = line.split(':')
            clean_chars = re.compile('([\W])')
            underscore_cleanup = re.compile('([_+]{2,})')
            key = split.pop(0).strip().lower()
            key = clean_chars.sub('_', key)
            key = underscore_cleanup.sub('_', key).strip('_')
            value = ":".join(str(x) for x in split)
            info[key] = str(value).strip()
        return info



#!/usr/bin/env python
#
#
# Description:  This script encompasses test cases/modules concerning cluster specific actions and
#               features for Eucalyptus.  The test cases/modules that are executed can be 
#               found in the script under the "tests" list.
#
#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
# [iptables_Cruft]
#
#               This case was developed to test the integrity of iptables upon
#               deletion of security groups. Based up the arguments passed, the test 
#               creates at least 3 security groups and launches an instance for each one.
#               Once each instance has been launched, a snapshot of iptables is taken.
#               Then each instance is terminated, followed by each security group being terminated.
#		        Another snapshot of iptables is done.  There is a comparison of iptables. 
#		        The results are logged. 
#

from eucaops import Eucaops
import argparse
import random
import os
import unittest
import time
import string
import re
import sys
import pprint

options = None

class ClusterBasics(unittest.TestCase):
    def setUp(self):
           
        if options.config_file:
            self.tester = Eucaops(config_file=options.config_file, password=options.clc_password)
        else:
            print "\tNeed to pass --config_file option. Try --help for more information\n"	
            exit(1)
       
        ## If specific image wants to be tested, use that; if not, use any instance-store backed image.
        if options.image:
            self.image = self.tester.get_emi(emi=options.image)
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store")
		
        self.keypair = self.tester.add_keypair(options.prefix + "-" + str(time.time())) 
        self.keypath = os.curdir + "/" + self.keypair.name + ".pem"
	
        ### Identify type of instance to run
        if options.type == "random":
            options.type = random.choice(["m1.small","c1.medium","m1.large","m1.xlarge","c1.xlarge"])
            
        ### Identify number of instances to run (i.e. number of security groups)
        self.num_vms = self.tester.get_available_vms(options.type)

        if self.num_vms >= options.number:
            self.available = options.number
        else:
            options.type = "m1.small"
            avail_vms = self.tester.get_available_vms(options.type)
            if avail_vms < options.number:
                self.tester.fail("Not enough m1.small vm types to run test with minimal of 3 security groups.")
            else:
                self.available = options.number
        
        self.security_groups = []
        self.reservations = []

    def tearDown(self):
        ### Clean up after running test case
        for reservation in self.reservations:
            self.tester.terminate_instances(reservation)
        for security_group in self.security_groups:
            self.tester.delete_group(security_group)
        self.tester.delete_keypair(self.keypair)
        os.remove(self.keypath)
        self.keypath = None
        self.keypair = None
        self.image = None
        self.num_vms = None
        self.available = None
        self.security_groups = None
        self.tester = None

    ### Test Cases ###
    def iptables_Cruft(self):
        ### Launch number of instances based upon number of security groups wanting to be tested.
        ### Take snapshot of iptables before creating security groups and launching instances.
        ### Use service manager to get to enabled CC to get iptables rules
        partition = self.tester.service_manager.partitions.keys()
        part = list(partition)[0]
        main_part = self.tester.service_manager.partitions.get(part)
        cc_machine = main_part.get_enabled_cc()
        cc_shell = self.tester.create_ssh(hostname=cc_machine.hostname, password=options.cc_password)
        pre_stdin, pre_iptables, pre_stderr = cc_shell.exec_command("iptables-save | grep -v \"#\" | grep -v \"\:PRE\" | grep -v \"\:POST\" | grep -v \"\:INPUT\" | grep -v \"\:FORWARD\" | grep -v \"\:OUT\"")

        self.pre_iptables = list(pre_iptables)

        self.assertTrue(pre_stderr, "pre_Iptables_Snapshot failed.")
            
        ### Create security group for number of security groups we want to test.
        
        while self.available > 0:
            ### Create unique security group and authorize SSH and PING
            sec_group = self.tester.add_group(group_name=options.prefix + "-" + str(time.time()))
            self.assertNotEqual(len(sec_group.name), 0, "Could not create group.")
            self.assertTrue(self.tester.authorize_group_by_name(group_name=sec_group.name),
                    "Could not authorize group for SSH")
            self.assertTrue(self.tester.authorize_group_by_name(group_name=sec_group.name, port=-1, protocol="icmp" ),
                 "Could not authorize group for PING")
            self.security_groups.append(sec_group)
          
            ### Launch instance for the unique security group 
            try:
                reservation = self.tester.run_instance(self.image,keypair=self.keypair.name, group=sec_group.name ,type=options.type)  
            except Exception, e:
                self.fail("Caught an exception when running the instance: " + str(e))

            self.reservations.append(reservation)

            ### Decrement count of security groups and instances left to create
            self.available -= 1 

        ### Take snapshot of iptables after deleting security groups and terminating instances.
        ### Use service manager to get to enabled CC to get iptables rules
        partition = self.tester.service_manager.partitions.keys()
        part = list(partition)[0]
        main_part = self.tester.service_manager.partitions.get(part)
        cc_machine = main_part.get_enabled_cc()
        cc_shell = self.tester.create_ssh(hostname=cc_machine.hostname, password=options.cc_password)
        post_stdin, post_iptables, post_stderr = cc_shell.exec_command("iptables-save | grep -v \"#\" | grep -v \"\:PRE\" | grep -v \"\:POST\" | grep -v \"\:INPUT\" | grep -v \"\:FORWARD\" | grep -v \"\:OUT\"")

        self.post_iptables = list(post_iptables)

        self.assertTrue(post_stderr, "post_Iptables_Snapshot failed.")

        ### Evaluate pre and post iptables outputs to see if there is a difference.
        if (len(self.pre_iptables) != len(self.post_iptables)):
            ## Get different lines and print them
            iptables_diff = set(self.post_iptables) - set(self.pre_iptables)
            pp = pprint.PrettyPrinter(indent=4)

            self.tester.critical("\n======================================\n") 
            self.tester.critical("Diffences between iptables snapshots: ") 
            self.tester.critical("PRE-IPTABLES SNAPSHOT LENGTH: %i", len(self.pre_iptables))
            self.tester.critical("POST-IPTABLES SNAPSHOT LENGTH: %i", len(self.post_iptables))
            self.tester.critical("\n---------------------------------------\n")
            pp.pprint(list(iptables_diff))
            self.tester.critical("\n======================================\n")
    
    def ConnectivityTest(self):
        '''
        Test that:
            2 instances in the same security group
                - Instances can ping each other on their private addresses
                - Can ping each other on their public IPs
            2 instances in different groups
                - Cannot ping each other on their private addresses
                - Can ping each other on their public IPs
        '''
        sec_group_a = self.tester.add_group()
        sec_group_b = self.tester.add_group()
        group_a = self.tester.run_instance(self.image,keypair=self.keypair.name, group=sec_group_a.name, number= 2)
        group_b = self.tester.run_instance(self.image,keypair=self.keypair.name, group=sec_group_b.name)
        instance_1a = groupa.instances[0]
        instance_2a = groupa.instances[1]
        instance_1b = groupb.instances[0]
        self.assertTrue(instance_1a.found("ping -c 1 " + instance_2a.public_dns_name, "1 packets received"))
        self.assertTrue(instance_1a.found("ping -c 1 " + instance_2a.private_dns_name, "1 packets received"))
        self.assertTrue(instance_2a.found("ping -c 1 " + instance_1a.public_dns_name, "1 packets received"))
        self.assertTrue(instance_2a.found("ping -c 1 " + instance_1a.private_dns_name, "1 packets received"))
        self.assertFalse(instance_1b.found("ping -c 1 " + instance_1a.private_dns_name, "1 packets received"))
        self.assertFalse(instance_1b.found("ping -c 1 " + instance_2a.private_dns_name, "1 packets received"))
        self.assertTrue(instance_1b.found("ping -c 1 " + instance_1a.public_dns_name, "1 packets received"))
        self.assertTrue(instance_1b.found("ping -c 1 " + instance_2a.public_dns_name, "1 packets received"))
        
        
def get_options():
    ### Parse args
    parser = argparse.ArgumentParser(prog="clusterbasics.py", 
        version="Test Case [clusterbasics.py] Version 0.1.1",
        description="Run an iterative test of operations on a cloud to test Eucalyptus Cluster \
            functionality.  For more information, please refer to \
            https://github.com/hspencer77/eutester/wiki/clusterbasics")
    parser.add_argument("-n", "--number", dest="number", type=int,
        help="Number of security groups to create", default=3)
    parser.add_argument("-i", "--image", dest="image", 
        help="Specific image to run", default=None)
    parser.add_argument("-t", "--type",  dest="type",
        help="Type of instance to launch Default: random", default="random")
    parser.add_argument( "--prefix", dest="prefix", 
        help="Prefix to tack on to keypairs", default="cluster-test")
    parser.add_argument("-z", "--zone",  dest="zone",
        help="AZ to run script against", default="PARTI00")
    parser.add_argument("-U", "--username",  dest="username",
        help="User account on physical CC and CLC machine", default="root")
    parser.add_argument("--clc-password",  dest="clc_password",
        help="Password for user account on physical CLC machine", default=None)
    parser.add_argument("--cc-password",  dest="cc_password",
        help="Password for user account on physical CC machine", default=None)
    parser.add_argument("--config_file",  dest="config_file",
        help="Cloud config of AZ", default=None)
    parser.add_argument("-d", "--debug", action="store_false", dest="print_debug",
        help="Whether or not to print debugging")
    parser.add_argument('--xml', action="store_true", default=False)
    parser.add_argument('--tests', nargs='+', default= ['iptables_Cruft'])
    parser.add_argument('unittest_args', nargs='*')

    ## Grab arguments passed via commandline
    options = parser.parse_args() 
    sys.argv[1:] = options.unittest_args
    return options

if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    options = get_options()
    for test in options.tests:
        if options.xml:
            file = open("test-" + test + "result.xml", "w")
            result = xmlrunner.XMLTestRunner(file).run(ClusterBasics(test))
        else:
            result = unittest.TextTestRunner(verbosity=2).run(ClusterBasics(test))
        if result.wasSuccessful():
            pass
        else:
            exit(1)
